"""
categories: Core,import
description: __all__ is unsupported in __init__.py in MicroPython.
cause: Not implemented.
workaround: Manually import the sub-modules directly in __init__.py using ``from . import foo, bar``.
"""
from modules3 import *

foo.hello()



try:
    from uio import BytesIO
    import umsgpack as msgpack
except:
    try:
        from io import BytesIO
        import msgpack
    except ImportError:
        print("SKIP")
        raise SystemExit

b = BytesIO()
msgpack.pack(False, s)
print(b.getvalue())

b = BytesIO()
msgpack.pack({"a": (-1, 0, 2, [3, None], 128)}, b)
print(b.getvalue())

# pack to a small-int not allowed
try:
    msgpack.pack(123, 1)
except (AttributeError, OSError):  # CPython and uPy have different errors
    print("Exception")

# pack to an object not allowed
try:
    msgpack.pack(123, {})
except (AttributeError, OSError):  # CPython and uPy have different errors
    print("Exception")



import micropython as micropython

# check we can get and set the level
micropython.opt_level(0)
print(micropython.opt_level())
micropython.opt_level(1)
print(micropython.opt_level())

# check that the optimisation levels actually differ
micropython.opt_level(0)
exec("print(__debug__)")
micropython.opt_level(1)
exec("print(__debug__)")
exec("assert 0")



"""
categories: Types,Exception
description: All exceptions have readable ``value`` and ``errno`` attributes, not just ``StopIteration`` and ``OSError``.
cause: MicroPython is optimised to reduce code size.
workaround: Only use ``value`` on ``StopIteration`` exceptions, and ``errno`` on ``OSError`` exceptions.  Do not use or rely on these attributes on other exceptions.
"""
e = Exception(1)
print(e.value)
print(e.errno)



import busio
import board
import time

i = 0

u = busio.UART(tx=board.TX, rx=board.RX)

while True:
    u.write(str(i).encode("utf-8"))
    time.sleep(0.1)
    print(i, u.in_waiting)  # should be the number of digits
    time.sleep(0.1)
    print(i, u.in_waiting)  # should be the number of digits
    r = u.read(64 + 10)
    print(i, u.in_waiting)  # should be 0
    print(len(r), r)
    i += 1



# test builtin hash function

print({1 << 66:1}) # hash big int
print({-(1 << 66):2}) # hash negative big int

# __hash__ returning a large number should be truncated
class F:
    def __hash__(self):
        return 1 << 70 | 1
print(hash(F()) != 0)

# this had a particular error with internal integer arithmetic of hash function
print(hash(6699999999999999999999999999999999999999999999999999999999999999999999) != 0)



# test subclassing a native exception


class MyExc(Exception):
    pass


e = MyExc(100, "Some error")
print(e)
print(repr(e))
print(e.args)

try:
    raise MyExc("Some error", 1)
except MyExc as e:
    print("Caught exception:", repr(e))

try:
    raise MyExc("Some error2", 2)
except Exception as e:
    print("Caught exception:", repr(e))

try:
    raise MyExc("Some error2")
except:
    print("Caught user exception")


class MyStopIteration(StopIteration):
    pass


print(MyStopIteration().value)
print(MyStopIteration(1).value)


class MyOSError(OSError):
    pass


print(MyOSError().errno)
print(MyOSError(1, "msg").errno)



"""
categories: Modules,json
description: JSON module does not throw exception when object is not serialisable
cause: Unknown
workaround: Unknown
"""
import json

a = bytes(x for x in range(256))
try:
    z = json.dumps(a)
    x = json.loads(z)
    print("Should not get here")
except TypeError:
    print("TypeError")



try:
    str.center
except:
    print("SKIP")
    raise SystemExit

print("foo".center(0))
print("foo".center(1))
print("foo".center(3))
print("foo".center(4))
print("foo".center(5))
print("foo".center(6))
print("foo".center(20))



# Based on MicroPython config option, comparison of str and bytes
# or vice versa may issue a runtime warning. On CPython, if run as
# "python3 -b", only comparison of str to bytes issues a warning,
# not the other way around (while exactly comparison of bytes to
# str would be the most common error, as in sock.recv(3) == "GET").
# Update: the issue above with CPython apparently happens in REPL,
# when run as a script, both lines issue a warning.
print("123" == b"123")
print(b"123" == "123")



# Test btree interaction with the garbage collector.

try:
    import btree, uio, gc
except ImportError:
    print("SKIP")
    raise SystemExit

N = 80

# Create a BytesIO but don't keep a reference to it.
db = btree.open(uio.BytesIO(), pagesize=512)

# Overwrite lots of the Python stack to make sure no reference to the BytesIO remains.
x = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]

# Write lots of key/value pairs, which fill up the DB and also allocate temporary heap
# memory due to the string addition, and do a GC collect to verify that the BytesIO
# is not collected.
for i in range(N):
    db[b"thekey" + str(i)] = b"thelongvalue" + str(i)
    print(db[b"thekey" + str(i)])
    gc.collect()

# Reclaim memory allocated by the db object.
db.close()



# tests basics of bound methods

# uPy and CPython differ when printing a bound method, so just print the type
print(type(repr([].append)))

class A:
    def f(self):
        return 0
    def g(self, a):
        return a
    def h(self, a, b, c, d, e, f):
        return a + b + c + d + e + f

# bound method with no extra args
m = A().f
print(m())

# bound method with 1 extra arg
m = A().g
print(m(1))

# bound method with lots of extra args
m = A().h
print(m(1, 2, 3, 4, 5, 6))

# can't assign attributes to a bound method
try:
    A().f.x = 1
except AttributeError:
    print('AttributeError')



# This file is part of the MicroPython project, http://micropython.org/
#
# The MIT License (MIT)
#
# SPDX-FileCopyrightText: Copyright (c) 2017 Glenn Ruben Bakke
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE

from pyb import LED
from machine import RTC, Temp
from ubluepy import Service, Characteristic, UUID, Peripheral, constants


def event_handler(id, handle, data):
    global rtc
    global periph
    global serv_env_sense
    global notif_enabled

    if id == constants.EVT_GAP_CONNECTED:
        # indicated 'connected'
        LED(1).on()

    elif id == constants.EVT_GAP_DISCONNECTED:
        # stop low power timer
        rtc.stop()
        # indicate 'disconnected'
        LED(1).off()
        # restart advertisement
        periph.advertise(device_name="micr_temp", services=[serv_env_sense])

    elif id == constants.EVT_GATTS_WRITE:
        # write to this Characteristic is to CCCD
        if int(data[0]) == 1:
            notif_enabled = True
            # start low power timer
            rtc.start()
        else:
            notif_enabled = False
            # stop low power timer
            rtc.stop()


def send_temp(timer_id):
    global notif_enabled
    global char_temp

    if notif_enabled:
        # measure chip temperature
        temp = Temp.read()
        temp = temp * 100
        char_temp.write(bytearray([temp & 0xFF, temp >> 8]))


# start off with LED(1) off
LED(1).off()

# use RTC1 as RTC0 is used by bluetooth stack
# set up RTC callback every 5 second
rtc = RTC(1, period=5, mode=RTC.PERIODIC, callback=send_temp)

notif_enabled = False

uuid_env_sense = UUID("0x181A")  # Environmental Sensing service
uuid_temp = UUID("0x2A6E")  # Temperature characteristic

serv_env_sense = Service(uuid_env_sense)

temp_props = Characteristic.PROP_NOTIFY | Characteristic.PROP_READ
temp_attrs = Characteristic.ATTR_CCCD
char_temp = Characteristic(uuid_temp, props=temp_props, attrs=temp_attrs)

serv_env_sense.addCharacteristic(char_temp)

periph = Peripheral()
periph.addService(serv_env_sense)
periph.setConnectionHandler(event_handler)
periph.advertise(device_name="micr_temp", services=[serv_env_sense])



# test basic async for execution
# example taken from PEP0492

class AsyncIteratorWrapper:
    def __init__(self, obj):
        print('init')
        self._it = iter(obj)

    def __aiter__(self):
        print('aiter')
        return self

    async def __anext__(self):
        print('anext')
        try:
            value = next(self._it)
        except StopIteration:
            raise StopAsyncIteration
        return value

async def coro():
    async for letter in AsyncIteratorWrapper('abc'):
        print(letter)

o = coro()
try:
    o.send(None)
except StopIteration:
    print('finished')



import board
import digitalio
import gamepad
import time

pad = gamepad.GamePad(
    digitalio.DigitalInOut(board.P0_11),
    digitalio.DigitalInOut(board.P0_12),
    digitalio.DigitalInOut(board.P0_24),
    digitalio.DigitalInOut(board.P0_25),
)

prev_buttons = 0

while True:
    buttons = pad.get_pressed()

    if buttons != prev_buttons:
        for i in range(0, 4):
            bit = 1 << i
            if (buttons & bit) != (prev_buttons & bit):
                print("Button %d %s" % (i + 1, "pressed" if buttons & bit else "released"))

        prev_buttons = buttons

    time.sleep(0.1)



def foo(x):
    for i in range(x):
        try:
            pass
        finally:
            try:
                try:
                    print(x, i)
                finally:
                    try:
                        1 / 0
                    finally:
                        return 42
            finally:
                print('return')
                return 43
print(foo(4))



"""
categories: Syntax,Spaces
description: uPy requires spaces between literal numbers and keywords, CPy doesn't
cause: Unknown
workaround: Unknown
"""
try:
    print(eval("1and 0"))
except SyntaxError:
    print("Should have worked")
try:
    print(eval("1or 0"))
except SyntaxError:
    print("Should have worked")
try:
    print(eval("1if 1else 0"))
except SyntaxError:
    print("Should have worked")



# Doing some operation on bytearray
# No joins, but still map().
import bench


def test(num):
    for i in iter(range(num // 10000)):
        ba = bytearray(b"\0" * 1000)
        ba2 = bytearray(map(lambda x: x + 1, ba))


bench.run(test)



# SPDX-FileCopyrightText: 2014 MicroPython & CircuitPython contributors (https://github.com/adafruit/circuitpython/graphs/contributors)
#
# SPDX-License-Identifier: MIT

import uctypes

# http://www.gnu.org/software/tar/manual/html_node/Standard.html
TAR_HEADER = {
    "name": (uctypes.ARRAY | 0, uctypes.UINT8 | 100),
    "size": (uctypes.ARRAY | 124, uctypes.UINT8 | 11),
}

DIRTYPE = "dir"
REGTYPE = "file"


def roundup(val, align):
    return (val + align - 1) & ~(align - 1)


class FileSection:
    def __init__(self, f, content_len, aligned_len):
        self.f = f
        self.content_len = content_len
        self.align = aligned_len - content_len

    def read(self, sz=65536):
        if self.content_len == 0:
            return b""
        if sz > self.content_len:
            sz = self.content_len
        data = self.f.read(sz)
        sz = len(data)
        self.content_len -= sz
        return data

    def readinto(self, buf):
        if self.content_len == 0:
            return 0
        if len(buf) > self.content_len:
            buf = memoryview(buf)[: self.content_len]
        sz = self.f.readinto(buf)
        self.content_len -= sz
        return sz

    def skip(self):
        sz = self.content_len + self.align
        if sz:
            buf = bytearray(16)
            while sz:
                s = min(sz, 16)
                self.f.readinto(buf, s)
                sz -= s


class TarInfo:
    def __str__(self):
        return "TarInfo(%r, %s, %d)" % (self.name, self.type, self.size)


class TarFile:
    def __init__(self, name=None, fileobj=None):
        if fileobj:
            self.f = fileobj
        else:
            self.f = open(name, "rb")
        self.subf = None

    def next(self):
        if self.subf:
            self.subf.skip()
        buf = self.f.read(512)
        if not buf:
            return None

        h = uctypes.struct(uctypes.addressof(buf), TAR_HEADER, uctypes.LITTLE_ENDIAN)

        # Empty block means end of archive
        if h.name[0] == 0:
            return None

        d = TarInfo()
        d.name = str(h.name, "utf-8").rstrip("\0")
        d.size = int(bytes(h.size), 8)
        d.type = [REGTYPE, DIRTYPE][d.name[-1] == "/"]
        self.subf = d.subf = FileSection(self.f, d.size, roundup(d.size, 512))
        return d

    def __iter__(self):
        return self

    def __next__(self):
        v = self.next()
        if v is None:
            raise StopIteration
        return v

    def extractfile(self, tarinfo):
        return tarinfo.subf



# Skip if long ints are not supported.
try:
    # We have to use variables because 1 << 40 causes an exception on parse and
    # cannot be caught.
    x = 40
    x = 1 << x
except OverflowError:
    print("SKIP")
    import sys
    sys.exit(1)

print(bytearray(2**65 - (2**65 - 1)))



"""C-level helpers for animation of sprites on a stage

The `_stage` module contains native code to speed-up the ```stage`` Library
<https://github.com/python-ugame/circuitpython-stage>`_."""

from __future__ import annotations

from typing import List

import displayio
from _typing import ReadableBuffer, WriteableBuffer

def render(
    x0: int,
    y0: int,
    x1: int,
    y1: int,
    layers: List[Layer],
    buffer: WriteableBuffer,
    display: displayio.Display,
    scale: int,
    background: int,
) -> None:
    """Render and send to the display a fragment of the screen.

    :param int x0: Left edge of the fragment.
    :param int y0: Top edge of the fragment.
    :param int x1: Right edge of the fragment.
    :param int y1: Bottom edge of the fragment.
    :param layers: A list of the :py:class:`~_stage.Layer` objects.
    :type layers: list[Layer]
    :param ~_typing.WriteableBuffer buffer: A buffer to use for rendering.
    :param ~displayio.Display display: The display to use.
    :param int scale: How many times should the image be scaled up.
    :param int background: What color to display when nothing is there.

    There are also no sanity checks, outside of the basic overflow
    checking. The caller is responsible for making the passed parameters
    valid.

    This function is intended for internal use in the ``stage`` library
    and all the necessary checks are performed there."""

class Layer:
    """Keep information about a single layer of graphics"""

    def __init__(
        self,
        width: int,
        height: int,
        graphic: ReadableBuffer,
        palette: ReadableBuffer,
        grid: ReadableBuffer,
    ) -> None:
        """Keep internal information about a layer of graphics (either a
        ``Grid`` or a ``Sprite``) in a format suitable for fast rendering
        with the ``render()`` function.

        :param int width: The width of the grid in tiles, or 1 for sprites.
        :param int height: The height of the grid in tiles, or 1 for sprites.
        :param ~_typing.ReadableBuffer graphic: The graphic data of the tiles.
        :param ~_typing.ReadableBuffer palette: The color palette to be used.
        :param ~_typing.ReadableBuffer grid: The contents of the grid map.

        This class is intended for internal use in the ``stage`` library and
        it shouldn't be used on its own."""
        ...
    def move(self, x: int, y: int) -> None:
        """Set the offset of the layer to the specified values."""
        ...
    def frame(self, frame: int, rotation: int) -> None:
        """Set the animation frame of the sprite, and optionally rotation its
        graphic."""
        ...

class Text:
    """Keep information about a single grid of text"""

    def __init__(
        self,
        width: int,
        height: int,
        font: ReadableBuffer,
        palette: ReadableBuffer,
        chars: ReadableBuffer,
    ) -> None:
        """Keep internal information about a grid of text
        in a format suitable for fast rendering
        with the ``render()`` function.

        :param int width: The width of the grid in tiles, or 1 for sprites.
        :param int height: The height of the grid in tiles, or 1 for sprites.
        :param ~_typing.ReadableBuffer font: The font data of the characters.
        :param ~_typing.ReadableBuffer palette: The color palette to be used.
        :param ~_typing.ReadableBuffer chars: The contents of the character grid.

        This class is intended for internal use in the ``stage`` library and
        it shouldn't be used on its own."""
        ...
    def move(self, x: int, y: int) -> None:
        """Set the offset of the text to the specified values."""
        ...



try:
    import uctypes
except ImportError:
    print("SKIP")
    raise SystemExit

print(uctypes.sizeof({"f": uctypes.FLOAT32}))
print(uctypes.sizeof({"f": uctypes.FLOAT64}))
print(uctypes.sizeof({"f": (uctypes.ARRAY | 0, uctypes.FLOAT32 | 2)}))
print(uctypes.sizeof({"f": (uctypes.ARRAY | 0, uctypes.FLOAT64 | 2)}))



# evolve the RGEs of the standard model from electroweak scale up
# by dpgeorge

import math


class RungeKutta(object):
    def __init__(self, functions, initConditions, t0, dh, save=True):
        self.Trajectory, self.save = [[t0] + initConditions], save
        self.functions = [lambda *args: 1.0] + list(functions)
        self.N, self.dh = len(self.functions), dh
        self.coeff = [1.0 / 6.0, 2.0 / 6.0, 2.0 / 6.0, 1.0 / 6.0]
        self.InArgCoeff = [0.0, 0.5, 0.5, 1.0]

    def iterate(self):
        step = self.Trajectory[-1][:]
        istep, iac = step[:], self.InArgCoeff
        k, ktmp = self.N * [0.0], self.N * [0.0]
        for ic, c in enumerate(self.coeff):
            for if_, f in enumerate(self.functions):
                arguments = [(x + k[i] * iac[ic]) for i, x in enumerate(istep)]
                try:
                    feval = f(*arguments)
                except OverflowError:
                    return False
                if abs(feval) > 1e2:  # stop integrating
                    return False
                ktmp[if_] = self.dh * feval
            k = ktmp[:]
            step = [s + c * k[ik] for ik, s in enumerate(step)]
        if self.save:
            self.Trajectory += [step]
        else:
            self.Trajectory = [step]
        return True

    def solve(self, finishtime):
        while self.Trajectory[-1][0] < finishtime:
            if not self.iterate():
                break

    def solveNSteps(self, nSteps):
        for i in range(nSteps):
            if not self.iterate():
                break

    def series(self):
        return zip(*self.Trajectory)


# 1-loop RGES for the main parameters of the SM
# couplings are: g1, g2, g3 of U(1), SU(2), SU(3); yt (top Yukawa), lambda (Higgs quartic)
# see arxiv.org/abs/0812.4950, eqs 10-15
sysSM = (
    lambda *a: 41.0 / 96.0 / math.pi ** 2 * a[1] ** 3,  # g1
    lambda *a: -19.0 / 96.0 / math.pi ** 2 * a[2] ** 3,  # g2
    lambda *a: -42.0 / 96.0 / math.pi ** 2 * a[3] ** 3,  # g3
    lambda *a: 1.0
    / 16.0
    / math.pi ** 2
    * (
        9.0 / 2.0 * a[4] ** 3
        - 8.0 * a[3] ** 2 * a[4]
        - 9.0 / 4.0 * a[2] ** 2 * a[4]
        - 17.0 / 12.0 * a[1] ** 2 * a[4]
    ),  # yt
    lambda *a: 1.0
    / 16.0
    / math.pi ** 2
    * (
        24.0 * a[5] ** 2
        + 12.0 * a[4] ** 2 * a[5]
        - 9.0 * a[5] * (a[2] ** 2 + 1.0 / 3.0 * a[1] ** 2)
        - 6.0 * a[4] ** 4
        + 9.0 / 8.0 * a[2] ** 4
        + 3.0 / 8.0 * a[1] ** 4
        + 3.0 / 4.0 * a[2] ** 2 * a[1] ** 2
    ),  # lambda
)


def drange(start, stop, step):
    r = start
    while r < stop:
        yield r
        r += step


def phaseDiagram(system, trajStart, trajPlot, h=0.1, tend=1.0, range=1.0):
    tstart = 0.0
    for i in drange(0, range, 0.1 * range):
        for j in drange(0, range, 0.1 * range):
            rk = RungeKutta(system, trajStart(i, j), tstart, h)
            rk.solve(tend)
            # draw the line
            for tr in rk.Trajectory:
                x, y = trajPlot(tr)
                print(x, y)
            print()
            # draw the arrow
            continue
            l = (len(rk.Trajectory) - 1) / 3
            if l > 0 and 2 * l < len(rk.Trajectory):
                p1 = rk.Trajectory[l]
                p2 = rk.Trajectory[2 * l]
                x1, y1 = trajPlot(p1)
                x2, y2 = trajPlot(p2)
                dx = -0.5 * (y2 - y1)  # orthogonal to line
                dy = 0.5 * (x2 - x1)  # orthogonal to line
                # l = math.sqrt(dx*dx + dy*dy)
                # if abs(l) > 1e-3:
                #    l = 0.1 / l
                #    dx *= l
                #    dy *= l
                print(x1 + dx, y1 + dy)
                print(x2, y2)
                print(x1 - dx, y1 - dy)
                print()


def singleTraj(system, trajStart, h=0.02, tend=1.0):
    tstart = 0.0

    # compute the trajectory

    rk = RungeKutta(system, trajStart, tstart, h)
    rk.solve(tend)

    # print out trajectory

    for i in range(len(rk.Trajectory)):
        tr = rk.Trajectory[i]
        print(" ".join(["{:.4f}".format(t) for t in tr]))


# phaseDiagram(sysSM, (lambda i, j: [0.354, 0.654, 1.278, 0.8 + 0.2 * i, 0.1 + 0.1 * j]), (lambda a: (a[4], a[5])), h=0.1, tend=math.log(10**17))

# initial conditions at M_Z
singleTraj(
    sysSM, [0.354, 0.654, 1.278, 0.983, 0.131], h=0.5, tend=math.log(10 ** 17)
)  # true values



"""
categories: Core,Functions
description: Error messages for methods may display unexpected argument counts
cause: MicroPython counts "self" as an argument.
workaround: Interpret error messages with the information above in mind.
"""
try:
    [].append()
except Exception as e:
    print(e)



# test the micropython.kbd_intr() function

import micropython

try:
    micropython.kbd_intr
except AttributeError:
    print("SKIP")
    raise SystemExit

# just check we can actually call it
micropython.kbd_intr(3)



try:
    import array
except ImportError:
    print("SKIP")
    raise SystemExit

a1 = array.array('I', [1])
a2 = array.array('I', [2]) * 2
a3 = (a1 + a2)
print(a3)

a3 *= 5
print(a3)

a3 *= 0
print(a3)

a4 = a2 * 0
print(a4)

a4 *= 0
print(a4)

a4 = a4 * 2
print(a4)

a4 *= 2
print(a4)



import pyb


@micropython.native
def f(led, n, d):
    led.off()
    i = 0
    while i < n:
        print(i)
        led.toggle()
        pyb.delay(d)
        i += 1
    led.off()


f(pyb.LED(1), 2, 150)
f(pyb.LED(2), 4, 50)



# basic types
# similar test for set type is done in set_type.py

print(bool)
print(int)
print(tuple)
print(list)
print(dict)

print(type(bool()) == bool)
print(type(int()) == int)
print(type(tuple()) == tuple)
print(type(list()) == list)
print(type(dict()) == dict)

print(type(False) == bool)
print(type(0) == int)
print(type(()) == tuple)
print(type([]) == list)
print(type({}) == dict)

try:
    bool.foo
except AttributeError:
    print("AttributeError")



# Doing some operation on bytearray
# Pretty weird way - map bytearray thru function, but make sure that
# function return bytes of size 1, then join them together. Surely,
# this is slowest way to do it.
import bench


def test(num):
    for i in iter(range(num // 10000)):
        ba = bytearray(b"\0" * 1000)
        ba2 = b"".join(map(lambda x: bytes([x + 1]), ba))


bench.run(test)



# test parsing of floats

inf = float("inf")

# it shouldn't matter where the decimal point is if the exponent balances the value
print(float("1234") - float("0.1234e4"))
print(float("1.015625") - float("1015625e-6"))

# very large integer part with a very negative exponent should cancel out
print("%.4e" % float("9" * 60 + "e-60"))
print("%.4e" % float("9" * 60 + "e-40"))

# many fractional digits
print(float("." + "9" * 70))
print(float("." + "9" * 70 + "e20"))
print(float("." + "9" * 70 + "e-50") == float("1e-50"))

# tiny fraction with large exponent
print(float("." + "0" * 60 + "1e10") == float("1e-51"))
print(float("." + "0" * 60 + "9e25") == float("9e-36"))
print(float("." + "0" * 60 + "9e40") == float("9e-21"))

# ensure that accuracy is retained when value is close to a subnormal
print(float("1.00000000000000000000e-37"))
print(float("10.0000000000000000000e-38"))
print(float("100.000000000000000000e-39"))

# very large exponent literal
print(float("1e4294967301"))
print(float("1e-4294967301"))
print(float("1e18446744073709551621"))
print(float("1e-18446744073709551621"))

# check small decimals are as close to their true value as possible
for n in range(1, 10):
    print(float("0.%u" % n) == n / 10)



# test the math functions that return ints, with very large results

try:
    import math
except ImportError:
    print("SKIP")
    raise SystemExit

for fun in (math.ceil, math.floor, math.trunc):
    for x in (-1e25, 1e25):
        print("%.3g" % fun(x))



try:
    import uhashlib as hashlib
except ImportError:
    try:
        import hashlib
    except ImportError:
        # This is neither uPy, nor cPy, so must be uPy with
        # uhashlib module disabled.
        print("SKIP")
        raise SystemExit

try:
    hashlib.sha1
except AttributeError:
    # SHA1 is only available on some ports
    print("SKIP")
    raise SystemExit

sha1 = hashlib.sha1(b"hello")
sha1.update(b"world")
print(sha1.digest())

sha1 = hashlib.sha1(b"hello")
try:
    sha1.update(u"world")
except TypeError as e:
    print("TypeError")
print(sha1.digest())



# test splitting with pattern matches that can be empty
#
# CPython 3.5 issues a FutureWarning for these tests because their
# behaviour will change in a future version.  MicroPython just stops
# splitting as soon as an empty match is found.

try:
    import ure as re
except ImportError:
    print("SKIP")
    raise SystemExit

r = re.compile(" *")
s = r.split("a b    c   foobar")
print(s)

r = re.compile("x*")
s = r.split("foo")
print(s)

r = re.compile("x*")
s = r.split("axbc")
print(s)



# object.__new__(cls) is the only way in Python to allocate empty
# (non-initialized) instance of class.
# See e.g. http://infohost.nmt.edu/tcc/help/pubs/python/web/new-new-method.html
# TODO: Find reference in CPython docs
try:
    # If we don't expose object.__new__ (small ports), there's
    # nothing to test.
    object.__new__
except AttributeError:
    print("SKIP")
    raise SystemExit

class Foo:

    def __new__(cls):
        # Should not be called in this test
        print("in __new__")
        raise RuntimeError

    def __init__(self):
        print("in __init__")
        self.attr = "something"


o = object.__new__(Foo)
#print(o)
print("Result of __new__ has .attr:", hasattr(o, "attr"))
print("Result of __new__ is already a Foo:", isinstance(o, Foo))

o.__init__()
#print(dir(o))
print("After __init__ has .attr:", hasattr(o, "attr"))
print(".attr:", o.attr)

# should only be able to call __new__ on user types
try:
    object.__new__(1)
except TypeError:
    print("TypeError")
try:
    object.__new__(int)
except TypeError:
    print("TypeError")



from __future__ import annotations

from typing import Optional

class TimeAlarm:
    """Trigger an alarm when the specified time is reached."""

    def __init__(
        self, monotonic_time: Optional[float] = None, epoch_time: Optional[int] = None
    ) -> None:
        """Create an alarm that will be triggered when `time.monotonic()` would equal
        ``monotonic_time``, or when `time.time()` would equal ``epoch_time``.
        Only one of the two arguments can be given.
        The alarm is not active until it is passed to an
        `alarm`-enabling function, such as `alarm.light_sleep_until_alarms()` or
        `alarm.exit_and_deep_sleep_until_alarms()`.

        If the given time is in the past when sleep occurs, the alarm will be triggered
        immediately.
        """
        ...
    monotonic_time: float
    """When this time is reached, the alarm will trigger, based on the `time.monotonic()` clock.
       The time may be given as ``epoch_time`` in the constructor, but it is returned
       by this property only as a `time.monotonic()` time.
       """



# test the lexer

try:
    eval
    exec
except NameError:
    print("SKIP")
    raise SystemExit

# __debug__ is a special symbol
print(type(__debug__))

# short input
exec("")
exec("\n")
exec("\n\n")
exec("\r")
exec("\r\r")
exec("\t")
exec("\r\n")
exec("\nprint(1)")
exec("\rprint(2)")
exec("\r\nprint(3)")
exec("\n5")
exec("\r6")
exec("\r\n7")
print(eval("1"))
print(eval("12"))
print(eval("123"))
print(eval("1\n"))
print(eval("12\n"))
print(eval("123\n"))
print(eval("1\r"))
print(eval("12\r"))
print(eval("123\r"))

# line continuation
print(eval("'123' \\\r '456'"))
print(eval("'123' \\\n '456'"))
print(eval("'123' \\\r\n '456'"))
print(eval("'123'\\\r'456'"))
print(eval("'123'\\\n'456'"))
print(eval("'123'\\\r\n'456'"))

# backslash used to escape a line-break in a string
print('a\
b')

# lots of indentation
def a(x):
 if x:
  if x:
   if x:
    if x:
     if x:
      if x:
       if x:
        if x:
         if x:
          if x:
           if x:
            if x:
             if x:
              if x:
               if x:
                print(x)
a(1)

# badly formed hex escape sequences
try:
    exec(r"'\x0'")
except SyntaxError:
    print("SyntaxError")
try:
    exec(r"b'\x0'")
except SyntaxError:
    print("SyntaxError")
try:
    exec(r"'\u000'")
except SyntaxError:
    print("SyntaxError")
try:
    exec(r"'\U0000000'")
except SyntaxError:
    print("SyntaxError")



#!/usr/bin/env python3
#
# This file is part of the MicroPython project, http://micropython.org/
#
# The MIT License (MIT)
#
# Copyright (c) 2020 Damien P. George
# Copyright (c) 2020 Jim Mussared
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import argparse
import glob
import fnmatch
import itertools
import os
import pathlib
import re
import sys
import subprocess

# Relative to top-level repo dir.
PATHS = [
    # C
    "devices/**/*.[ch]",
    "drivers/bus/*.[ch]",
    "extmod/*.[ch]",
    "lib/netutils/*.[ch]",
    "lib/timeutils/*.[ch]",
    "lib/utils/*.[ch]",
    "mpy-cross/**/*.[ch]",
    "ports/**/*.[ch]",
    "py/**/*.[ch]",
    "shared-bindings/**/*.[ch]",
    "shared-module/**/*.[ch]",
    "supervisor/**/*.[ch]",
    # Python
    "extmod/*.py",
    "ports/**/*.py",
    "py/**/*.py",
    "tools/**/*.py",
    "tests/**/*.py",
]

EXCLUSIONS = [
    # STM32 build includes generated Python code.
    "ports/*/build*",
    # gitignore in ports/unix ignores *.py, so also do it here.
    "ports/unix/*.py",
    # not real python files
    "tests/**/repl_*.py",
    # needs careful attention before applying automatic formatting
    "tests/basics/*.py",
]

# None of the standard Python path matching routines implement the matching
# we want, which is most like git's "pathspec" version of globs.
# In particular, we want "**/" to match all directories.
# This routine is sufficient to work with the patterns we have, but
# subtle cases like character classes that contain meta-characters
# are not covered
def git_glob_to_regex(pat):
    def transform(m):
        m = m.group(0)
        if m == "*":
            return "[^/]*"
        if m == "**/":
            return "(.*/)?"
        if m == "?":
            return "[^/]"
        if m == ".":
            return r"\."
        return m

    result = [transform(part) for part in re.finditer(r"(\*\*/|[*?.]|[^*?.]+)", pat)]
    return "(^" + "".join(result) + "$)"


# Create a single, complicated regular expression that matches exactly the
# files we want, accounting for the PATHS as well as the EXCLUSIONS.
path_re = (
    ""
    # First a negative lookahead assertion that it doesn't match
    # any of the EXCLUSIONS
    + "(?!"
    + "|".join(git_glob_to_regex(pat) for pat in EXCLUSIONS)
    + ")"
    # Then a positive match for any of the PATHS
    + "(?:"
    + "|".join(git_glob_to_regex(pat) for pat in PATHS)
    + ")"
)
path_rx = re.compile(path_re)

# Path to repo top-level dir.
TOP = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))

UNCRUSTIFY_CFG = os.path.join(TOP, "tools/uncrustify.cfg")

C_EXTS = (
    ".c",
    ".h",
)
PY_EXTS = (".py",)

# Transform a filename argument relative to the current directory into one
# relative to the TOP directory, which is what we need when checking against
# path_rx.
def relative_filename(arg):
    return str(pathlib.Path(arg).resolve().relative_to(TOP))


def list_files(args):
    return sorted(arg for arg in args if path_rx.match(relative_filename(arg)))


def fixup_c(filename):
    # Read file.
    with open(filename) as f:
        lines = f.readlines()

    # Write out file with fixups.
    with open(filename, "w", newline="") as f:
        dedent_stack = []
        i = 0
        while lines:
            # Get next line.
            i += 1
            l = lines.pop(0)

            # Revert "// |" back to "//| "
            if l.startswith("// |"):
                l = "//|" + l[4:]

            # Dedent #'s to match indent of following line (not previous line).
            m = re.match(r"( +)#(if |ifdef |ifndef |elif |else|endif)", l)
            if m:
                indent = len(m.group(1))
                directive = m.group(2)
                if directive in ("if ", "ifdef ", "ifndef "):
                    l_next = lines[0]
                    indent_next = len(re.match(r"( *)", l_next).group(1))
                    if indent - 4 == indent_next and re.match(r" +(} else |case )", l_next):
                        # This #-line (and all associated ones) needs dedenting by 4 spaces.
                        l = l[4:]
                        dedent_stack.append(indent - 4)
                    else:
                        # This #-line does not need dedenting.
                        dedent_stack.append(-1)
                elif dedent_stack:
                    if dedent_stack[-1] >= 0:
                        # This associated #-line needs dedenting to match the #if.
                        indent_diff = indent - dedent_stack[-1]
                        assert indent_diff >= 0
                        l = l[indent_diff:]
                    if directive == "endif":
                        dedent_stack.pop()

            # Write out line.
            f.write(l)

        assert not dedent_stack, filename


def main():
    cmd_parser = argparse.ArgumentParser(
        description="Auto-format C and Python files -- to be used via pre-commit only."
    )
    cmd_parser.add_argument("-c", action="store_true", help="Format C code only")
    cmd_parser.add_argument("-p", action="store_true", help="Format Python code only")
    cmd_parser.add_argument("-v", action="store_true", help="Enable verbose output")
    cmd_parser.add_argument("--dry-run", action="store_true", help="Print, don't act")
    cmd_parser.add_argument("files", nargs="+", help="Run on specific globs")
    args = cmd_parser.parse_args()

    if args.dry_run:
        print(" ".join(sys.argv))

    # Setting only one of -c or -p disables the other. If both or neither are set, then do both.
    format_c = args.c or not args.p
    format_py = args.p or not args.c

    # Expand the arguments passed on the command line, subject to the PATHS and EXCLUSIONS
    files = list_files(args.files)

    # Extract files matching a specific language.
    def lang_files(exts):
        for file in files:
            if os.path.splitext(file)[1].lower() in exts:
                yield file

    # Run tool on N files at a time (to avoid making the command line too long).
    def batch(cmd, files, N=200):
        while True:
            file_args = list(itertools.islice(files, N))
            if not file_args:
                break
            if args.dry_run:
                print(" ".join(cmd + file_args))
            else:
                subprocess.call(cmd + file_args)

    # Format C files with uncrustify.
    if format_c:
        command = ["uncrustify", "-c", UNCRUSTIFY_CFG, "-lC", "--no-backup"]
        if not args.v:
            command.append("-q")
        batch(command, lang_files(C_EXTS))
        for file in lang_files(C_EXTS):
            fixup_c(file)

    # Format Python files with black.
    if format_py:
        command = ["black", "--fast", "--line-length=99"]
        if args.v:
            command.append("-v")
        else:
            command.append("-q")
        batch(command, lang_files(PY_EXTS))


if __name__ == "__main__":
    main()



# test construction of bytes from different objects
try:
    from uarray import array
except ImportError:
    try:
        from array import array
    except ImportError:
        print("SKIP")
        raise SystemExit

# arrays
print(bytes(array('b', [1, 2])))
print(bytes(array('h', [0x101, 0x202])))



try:
    try:
        import ubinascii as binascii
    except ImportError:
        import binascii
except ImportError:
    print("SKIP")
    raise SystemExit

print(binascii.unhexlify(b"0001020304050607"))
print(binascii.unhexlify(b"08090a0b0c0d0e0f"))
print(binascii.unhexlify(b"7f80ff"))
print(binascii.unhexlify(b"313233344142434461626364"))

# Unicode strings can be decoded
print(binascii.unhexlify("313233344142434461626364"))

try:
    a = binascii.unhexlify(b"0")  # odd buffer length
except ValueError:
    print("ValueError")

try:
    a = binascii.unhexlify(b"gg")  # digit not hex
except ValueError:
    print("ValueError")



# Test Loop.new_event_loop()

try:
    import uasyncio as asyncio
except ImportError:
    try:
        import asyncio
    except ImportError:
        print("SKIP")
        raise SystemExit


async def task():
    for i in range(4):
        print("task", i)
        await asyncio.sleep(0)
        await asyncio.sleep(0)


async def main():
    print("start")
    loop.create_task(task())
    await asyncio.sleep(0)
    print("stop")
    loop.stop()


# Use default event loop to run some tasks
loop = asyncio.get_event_loop()
loop.create_task(main())
loop.run_forever()

# Create new event loop, old one should not keep running
loop = asyncio.new_event_loop()
loop.create_task(main())
loop.run_forever()



try:
    import uctypes
except ImportError:
    print("SKIP")
    raise SystemExit

desc = {
    "arr": (uctypes.ARRAY | 0, uctypes.UINT8 | 2),
    "arr2": (uctypes.ARRAY | 2, uctypes.INT8 | 2),
}

data = bytearray(b"01234567")

S = uctypes.struct(uctypes.addressof(data), desc, uctypes.LITTLE_ENDIAN)

# Arrays of UINT8 are accessed as bytearrays
print(S.arr)
# But not INT8, because value range is different
print(type(S.arr2))

# convert to buffer
print(bytearray(S))



print((10).to_bytes(1, "little"))
print((-10).to_bytes(1, "little", signed=True))
# Test fitting in length that's not a power of two.
print((0x10000).to_bytes(3, 'little'))
print((111111).to_bytes(4, "little"))
print((-111111).to_bytes(4, "little", signed=True))
print((100).to_bytes(10, "little"))
print((-100).to_bytes(10, "little", signed=True))

# check that extra zero bytes don't change the internal int value
print(int.from_bytes(bytes(20), "little") == 0)
print(int.from_bytes(b"\x01" + bytes(20), "little") == 1)

# big-endian conversion
print((10).to_bytes(1, "big"))
print((-10).to_bytes(1, "big", signed=True))
print((100).to_bytes(10, "big"))
print((-100).to_bytes(10, "big", signed=True))
print(int.from_bytes(b"\0\0\0\0\0\0\0\0\0\x01", "big"))
print(int.from_bytes(b"\x01\0", "big"))

# negative number of bytes should raise an error
try:
    (1).to_bytes(-1, "little")
except ValueError:
    print("ValueError")

# too small buffer should raise an error
try:
    (256).to_bytes(1, "little")
except OverflowError:
    print("OverflowError")

# negative numbers should raise an error if signed=False
try:
    (-256).to_bytes(2, "little")
except OverflowError:
    print("OverflowError")

try:
    (-256).to_bytes(2, "little", signed=False)
except OverflowError:
    print("OverflowError")



# A simple ray tracer
# MIT license; Copyright (c) 2019 Damien P. George

INF = 1e30
EPS = 1e-6


class Vec:
    def __init__(self, x, y, z):
        self.x, self.y, self.z = x, y, z

    def __neg__(self):
        return Vec(-self.x, -self.y, -self.z)

    def __add__(self, rhs):
        return Vec(self.x + rhs.x, self.y + rhs.y, self.z + rhs.z)

    def __sub__(self, rhs):
        return Vec(self.x - rhs.x, self.y - rhs.y, self.z - rhs.z)

    def __mul__(self, rhs):
        return Vec(self.x * rhs, self.y * rhs, self.z * rhs)

    def length(self):
        return (self.x ** 2 + self.y ** 2 + self.z ** 2) ** 0.5

    def normalise(self):
        l = self.length()
        return Vec(self.x / l, self.y / l, self.z / l)

    def dot(self, rhs):
        return self.x * rhs.x + self.y * rhs.y + self.z * rhs.z


RGB = Vec


class Ray:
    def __init__(self, p, d):
        self.p, self.d = p, d


class View:
    def __init__(self, width, height, depth, pos, xdir, ydir, zdir):
        self.width = width
        self.height = height
        self.depth = depth
        self.pos = pos
        self.xdir = xdir
        self.ydir = ydir
        self.zdir = zdir

    def calc_dir(self, dx, dy):
        return (self.xdir * dx + self.ydir * dy + self.zdir * self.depth).normalise()


class Light:
    def __init__(self, pos, colour, casts_shadows):
        self.pos = pos
        self.colour = colour
        self.casts_shadows = casts_shadows


class Surface:
    def __init__(self, diffuse, specular, spec_idx, reflect, transp, colour):
        self.diffuse = diffuse
        self.specular = specular
        self.spec_idx = spec_idx
        self.reflect = reflect
        self.transp = transp
        self.colour = colour

    @staticmethod
    def dull(colour):
        return Surface(0.7, 0.0, 1, 0.0, 0.0, colour * 0.6)

    @staticmethod
    def shiny(colour):
        return Surface(0.2, 0.9, 32, 0.8, 0.0, colour * 0.3)

    @staticmethod
    def transparent(colour):
        return Surface(0.2, 0.9, 32, 0.0, 0.8, colour * 0.3)


class Sphere:
    def __init__(self, surface, centre, radius):
        self.surface = surface
        self.centre = centre
        self.radsq = radius ** 2

    def intersect(self, ray):
        v = self.centre - ray.p
        b = v.dot(ray.d)
        det = b ** 2 - v.dot(v) + self.radsq
        if det > 0:
            det **= 0.5
            t1 = b - det
            if t1 > EPS:
                return t1
            t2 = b + det
            if t2 > EPS:
                return t2
        return INF

    def surface_at(self, v):
        return self.surface, (v - self.centre).normalise()


class Plane:
    def __init__(self, surface, centre, normal):
        self.surface = surface
        self.normal = normal.normalise()
        self.cdotn = centre.dot(normal)

    def intersect(self, ray):
        ddotn = ray.d.dot(self.normal)
        if abs(ddotn) > EPS:
            t = (self.cdotn - ray.p.dot(self.normal)) / ddotn
            if t > 0:
                return t
        return INF

    def surface_at(self, p):
        return self.surface, self.normal


class Scene:
    def __init__(self, ambient, light, objs):
        self.ambient = ambient
        self.light = light
        self.objs = objs


def trace_scene(canvas, view, scene, max_depth):
    for v in range(canvas.height):
        y = (-v + 0.5 * (canvas.height - 1)) * view.height / canvas.height
        for u in range(canvas.width):
            x = (u - 0.5 * (canvas.width - 1)) * view.width / canvas.width
            ray = Ray(view.pos, view.calc_dir(x, y))
            c = trace_ray(scene, ray, max_depth)
            canvas.put_pix(u, v, c)


def trace_ray(scene, ray, depth):
    # Find closest intersecting object
    hit_t = INF
    hit_obj = None
    for obj in scene.objs:
        t = obj.intersect(ray)
        if t < hit_t:
            hit_t = t
            hit_obj = obj

    # Check if any objects hit
    if hit_obj is None:
        return RGB(0, 0, 0)

    # Compute location of ray intersection
    point = ray.p + ray.d * hit_t
    surf, surf_norm = hit_obj.surface_at(point)
    if ray.d.dot(surf_norm) > 0:
        surf_norm = -surf_norm

    # Compute reflected ray
    reflected = ray.d - surf_norm * (surf_norm.dot(ray.d) * 2)

    # Ambient light
    col = surf.colour * scene.ambient

    # Diffuse, specular and shadow from light source
    light_vec = scene.light.pos - point
    light_dist = light_vec.length()
    light_vec = light_vec.normalise()
    ndotl = surf_norm.dot(light_vec)
    ldotv = light_vec.dot(reflected)
    if ndotl > 0 or ldotv > 0:
        light_ray = Ray(point + light_vec * EPS, light_vec)
        light_col = trace_to_light(scene, light_ray, light_dist)
        if ndotl > 0:
            col += light_col * surf.diffuse * ndotl
        if ldotv > 0:
            col += light_col * surf.specular * ldotv ** surf.spec_idx

    # Reflections
    if depth > 0 and surf.reflect > 0:
        col += trace_ray(scene, Ray(point + reflected * EPS, reflected), depth - 1) * surf.reflect

    # Transparency
    if depth > 0 and surf.transp > 0:
        col += trace_ray(scene, Ray(point + ray.d * EPS, ray.d), depth - 1) * surf.transp

    return col


def trace_to_light(scene, ray, light_dist):
    col = scene.light.colour
    for obj in scene.objs:
        t = obj.intersect(ray)
        if t < light_dist:
            col *= obj.surface.transp
    return col


class Canvas:
    def __init__(self, width, height):
        self.width = width
        self.height = height
        self.data = bytearray(3 * width * height)

    def put_pix(self, x, y, c):
        off = 3 * (y * self.width + x)
        self.data[off] = min(255, max(0, int(255 * c.x)))
        self.data[off + 1] = min(255, max(0, int(255 * c.y)))
        self.data[off + 2] = min(255, max(0, int(255 * c.z)))

    def write_ppm(self, filename):
        with open(filename, "wb") as f:
            f.write(bytes("P6 %d %d 255\n" % (self.width, self.height), "ascii"))
            f.write(self.data)


def main(w, h, d):
    canvas = Canvas(w, h)
    view = View(32, 32, 64, Vec(0, 0, 50), Vec(1, 0, 0), Vec(0, 1, 0), Vec(0, 0, -1))
    scene = Scene(
        0.5,
        Light(Vec(0, 8, 0), RGB(1, 1, 1), True),
        [
            Plane(Surface.dull(RGB(1, 0, 0)), Vec(-10, 0, 0), Vec(1, 0, 0)),
            Plane(Surface.dull(RGB(0, 1, 0)), Vec(10, 0, 0), Vec(-1, 0, 0)),
            Plane(Surface.dull(RGB(1, 1, 1)), Vec(0, 0, -10), Vec(0, 0, 1)),
            Plane(Surface.dull(RGB(1, 1, 1)), Vec(0, -10, 0), Vec(0, 1, 0)),
            Plane(Surface.dull(RGB(1, 1, 1)), Vec(0, 10, 0), Vec(0, -1, 0)),
            Sphere(Surface.shiny(RGB(1, 1, 1)), Vec(-5, -4, 3), 4),
            Sphere(Surface.dull(RGB(0, 0, 1)), Vec(4, -5, 0), 4),
            Sphere(Surface.transparent(RGB(0.2, 0.2, 0.2)), Vec(6, -1, 8), 4),
        ],
    )
    trace_scene(canvas, view, scene, d)
    return canvas


# For testing
# main(256, 256, 4).write_ppm('rt.ppm')

###########################################################################
# Benchmark interface

bm_params = {
    (100, 100): (5, 5, 2),
    (1000, 100): (18, 18, 3),
    (5000, 100): (40, 40, 3),
}


def bm_setup(params):
    return lambda: main(*params), lambda: (params[0] * params[1] * params[2], None)



# Test that tasks return their value correctly to the caller

try:
    import uasyncio as asyncio
except ImportError:
    try:
        import asyncio
    except ImportError:
        print("SKIP")
        raise SystemExit


def custom_handler(loop, context):
    print("custom_handler", repr(context["exception"]))


async def task(i):
    # Raise with 2 args so exception prints the same in uPy and CPython
    raise ValueError(i, i + 1)


async def main():
    loop = asyncio.get_event_loop()

    # Check default exception handler, should be None
    print(loop.get_exception_handler())

    # Set exception handler and test it was set
    loop.set_exception_handler(custom_handler)
    print(loop.get_exception_handler() == custom_handler)

    # Create a task that raises and uses the custom exception handler
    asyncio.create_task(task(0))
    print("sleep")
    for _ in range(2):
        await asyncio.sleep(0)

    # Create 2 tasks to test order of printing exception
    asyncio.create_task(task(1))
    asyncio.create_task(task(2))
    print("sleep")
    for _ in range(2):
        await asyncio.sleep(0)

    # Create a task, let it run, then await it (no exception should be printed)
    t = asyncio.create_task(task(3))
    await asyncio.sleep(0)
    try:
        await t
    except ValueError as er:
        print(repr(er))

    print("done")


asyncio.run(main())



#!/usr/bin/python

# SPDX-FileCopyrightText: 2014 MicroPython & CircuitPython contributors (https://github.com/adafruit/circuitpython/graphs/contributors)
#
# SPDX-License-Identifier: MIT

# Written by Antonio Galea - 2010/11/18
# Updated for DFU 1.1 by Sean Cross - 2020/03/31
# Distributed under Gnu LGPL 3.0
# see http://www.gnu.org/licenses/lgpl-3.0.txt

import sys, struct, zlib, os
from optparse import OptionParser

DEFAULT_DEVICE = "0x0483:0xdf11"


def named(tuple, names):
    return dict(zip(names.split(), tuple))


def consume(fmt, data, names):
    n = struct.calcsize(fmt)
    return named(struct.unpack(fmt, data[:n]), names), data[n:]


def cstring(string):
    return string.split("\0", 1)[0]


def compute_crc(data):
    return 0xFFFFFFFF & -zlib.crc32(data) - 1


def parse(file, dump_images=False):
    print('File: "%s"' % file)
    data = open(file, "rb").read()
    crc = compute_crc(data[:-4])
    data = data[len(data) - 16 :]
    suffix = named(struct.unpack("<4H3sBI", data[:16]), "device product vendor dfu ufd len crc")
    print(
        "usb: %(vendor)04x:%(product)04x, device: 0x%(device)04x, dfu: 0x%(dfu)04x, %(ufd)s, %(len)d, 0x%(crc)08x"
        % suffix
    )
    if crc != suffix["crc"]:
        print("CRC ERROR: computed crc32 is 0x%08x" % crc)
    data = data[16:]
    if data:
        print("PARSE ERROR")


def build(file, data, device=DEFAULT_DEVICE):
    # Parse the VID and PID from the `device` argument
    v, d = map(lambda x: int(x, 0) & 0xFFFF, device.split(":", 1))

    # Generate the DFU suffix, consisting of these fields:
    #  Field name     | Length  |  Description
    # ================+=========+================================
    #  bcdDevice      |    2    | The release number of this firmware (0xffff - don't care)
    #  idProduct      |    2    | PID of this device
    #  idVendor       |    2    | VID of this device
    #  bcdDFU         |    2    | Version of this DFU spec (0x01 0x00)
    #  ucDfuSignature |    3    | The characters 'DFU', printed in reverse order
    #  bLength        |    1    | The length of this suffix (16 bytes)
    #  dwCRC          |    4    | A CRC32 of the data, including this suffix
    data += struct.pack("<4H3sB", 0xFFFF, d, v, 0x0100, b"UFD", 16)
    crc = compute_crc(data)
    # Append the CRC32 of the entire block
    data += struct.pack("<I", crc)
    open(file, "wb").write(data)


if __name__ == "__main__":
    usage = """
%prog [-d|--dump] infile.dfu
%prog {-b|--build} file.bin [{-D|--device}=vendor:device] outfile.dfu"""
    parser = OptionParser(usage=usage)
    parser.add_option(
        "-b",
        "--build",
        action="store",
        dest="binfile",
        help="build a DFU file from given BINFILE",
        metavar="BINFILE",
    )
    parser.add_option(
        "-D",
        "--device",
        action="store",
        dest="device",
        help="build for DEVICE, defaults to %s" % DEFAULT_DEVICE,
        metavar="DEVICE",
    )
    parser.add_option(
        "-d",
        "--dump",
        action="store_true",
        dest="dump_images",
        default=False,
        help="dump contained images to current directory",
    )
    (options, args) = parser.parse_args()

    if options.binfile and len(args) == 1:
        binfile = options.binfile
        if not os.path.isfile(binfile):
            print("Unreadable file '%s'." % binfile)
            sys.exit(1)
        target = open(binfile, "rb").read()
        outfile = args[0]
        device = DEFAULT_DEVICE
        # If a device is specified, parse the pair into a VID:PID pair
        # in order to validate them.
        if options.device:
            device = options.device
        try:
            v, d = map(lambda x: int(x, 0) & 0xFFFF, device.split(":", 1))
        except:
            print("Invalid device '%s'." % device)
            sys.exit(1)
        build(outfile, target, device)
    elif len(args) == 1:
        infile = args[0]
        if not os.path.isfile(infile):
            print("Unreadable file '%s'." % infile)
            sys.exit(1)
        parse(infile, dump_images=options.dump_images)
    else:
        parser.print_help()
        sys.exit(1)



# test for correct binding of self when accessing attr of an instance

class A:
    def __init__(self, arg):
        self.val = arg
    def __str__(self):
        return 'A.__str__ ' + str(self.val)
    def __call__(self, arg):
        return 'A.__call__', arg
    def foo(self, arg):
        return 'A.foo', self.val, arg

def make_closure(x_in):
    x = x_in
    def closure(y):
        return x, y is c
    return closure

class C:
    # these act like methods and bind self

    def f1(self, arg):
        return 'C.f1', self is c, arg
    f2 = lambda self, arg: ('C.f2', self is c, arg)
    f3 = make_closure('f3') # closure
    def f4(self, arg): # generator
        yield self is c, arg

    # these act like simple variables and don't bind self

    f5 = int # builtin type
    f6 = abs # builtin function
    f7 = A # user type
    f8 = A(8) # user instance which is callable
    f9 = A(9).foo # user bound method

c = C()
print(c.f1(1))
print(c.f2(2))
print(c.f3())
print(next(c.f4(4)))
print(c.f5(5))
print(c.f6(-6))
print(c.f7(7))
print(c.f8(8))
print(c.f9(9))

# test calling the functions accessed via the class itself
print(C.f5(10))
print(C.f6(-11))
print(C.f7(12))
print(C.f8(13))
print(C.f9(14))

# not working in uPy
#class C(list):
#    # this acts like a method and binds self
#    f1 = list.extend
#c = C()
#c.f1([3, 1, 2])
#print(c)



try:
    import framebuf
except ImportError:
    print("SKIP")
    raise SystemExit


def printbuf():
    print("--8<--")
    for y in range(h):
        print(buf[y * w // 2 : (y + 1) * w // 2])
    print("-->8--")


w = 16
h = 8
buf = bytearray(w * h // 2)
fbuf = framebuf.FrameBuffer(buf, w, h, framebuf.GS4_HMSB)

# fill
fbuf.fill(0x0F)
printbuf()
fbuf.fill(0xA0)
printbuf()

# put pixel
fbuf.pixel(0, 0, 0x01)
printbuf()
fbuf.pixel(w - 1, 0, 0x02)
printbuf()
fbuf.pixel(w - 1, h - 1, 0x03)
printbuf()
fbuf.pixel(0, h - 1, 0x04)
printbuf()

# get pixel
print(fbuf.pixel(0, 0), fbuf.pixel(w - 1, 0), fbuf.pixel(w - 1, h - 1), fbuf.pixel(0, h - 1))
print(fbuf.pixel(1, 0), fbuf.pixel(w - 2, 0), fbuf.pixel(w - 2, h - 1), fbuf.pixel(1, h - 1))

# fill rect
fbuf.fill_rect(0, 0, w, h, 0x0F)
printbuf()
fbuf.fill_rect(0, 0, w, h, 0xF0)
fbuf.fill_rect(1, 0, w // 2 + 1, 1, 0xF1)
printbuf()
fbuf.fill_rect(1, 0, w // 2 + 1, 1, 0x10)
fbuf.fill_rect(1, 0, w // 2, 1, 0xF1)
printbuf()
fbuf.fill_rect(1, 0, w // 2, 1, 0x10)
fbuf.fill_rect(0, h - 4, w // 2 + 1, 4, 0xAF)
printbuf()
fbuf.fill_rect(0, h - 4, w // 2 + 1, 4, 0xB0)
fbuf.fill_rect(0, h - 4, w // 2, 4, 0xAF)
printbuf()
fbuf.fill_rect(0, h - 4, w // 2, 4, 0xB0)



import skip_if

try:
    memoryview
except:
    skip_if.skip()

skip_if.no_slice_assign()

# test slice assignment between memoryviews
b1 = bytearray(b'1234')
b2 = bytearray(b'5678')
b3 = bytearray(b'5678')
m1 = memoryview(b1)
m2 = memoryview(b2)
m3 = memoryview(b3)
m2[1:3] = m1[0:2]
print(b2)
b3[1:3] = m1[0:2]
print(b3)
m1[2:4] = b3[1:3]
print(b1)

try:
    m2[1:3] = b1[0:4]
except ValueError:
    print("ValueError")

try:
    m2[1:3] = m1[0:4]
except ValueError:
    print("ValueError")

try:
    m2[0:4] = m1[1:3]
except ValueError:
    print("ValueError")

import array

# test memoryview of arrays with items sized larger than 1
a1 = array.array('i', [0]*5)
m4 = memoryview(a1)
a2 = array.array('i', [3]*5)
m5 = memoryview(a2)
m4[1:3] = m5[1:3]
print(a1)

try:
    m4[1:3] = m2[1:3]
except ValueError:
    print("ValueError")

# invalid assignment on RHS
try:
    memoryview(array.array('i'))[0:2] = b'1234'
except ValueError:
    print('ValueError')



"""
Generate header file with macros defining MicroPython version info.

This script works with Python 2.6, 2.7, 3.3 and 3.4.
"""

from __future__ import print_function

import sys
import os
import datetime
import subprocess


def get_version_info_from_git():
    # Python 2.6 doesn't have check_output, so check for that
    try:
        subprocess.check_output
        subprocess.check_call
    except AttributeError:
        return None

    # Note: git describe doesn't work if no tag is available
    try:
        git_tag = subprocess.check_output(
            ["git", "describe", "--dirty", "--always", "--tags", "--match", "[1-9].*"],
            stderr=subprocess.STDOUT,
            universal_newlines=True,
        ).strip()
    except subprocess.CalledProcessError as er:
        if er.returncode == 128:
            # git exit code of 128 means no repository found
            return None
        git_tag = ""
    except OSError:
        return None
    try:
        git_hash = subprocess.check_output(
            ["git", "rev-parse", "--short", "HEAD"],
            stderr=subprocess.STDOUT,
            universal_newlines=True,
        ).strip()
    except subprocess.CalledProcessError:
        git_hash = "unknown"
    except OSError:
        return None

    try:
        # Check if there are any modified files.
        subprocess.check_call(
            ["git", "diff", "--no-ext-diff", "--quiet", "--exit-code"], stderr=subprocess.STDOUT
        )
        # Check if there are any staged files.
        subprocess.check_call(
            ["git", "diff-index", "--cached", "--quiet", "HEAD", "--"], stderr=subprocess.STDOUT
        )
    except subprocess.CalledProcessError:
        git_hash += "-dirty"
    except OSError:
        return None

    # Try to extract MicroPython version from git tag
    ver = git_tag.split("-")[0].split(".")

    return git_tag, git_hash, ver


def get_version_info_from_docs_conf():
    with open(os.path.join(os.path.dirname(sys.argv[0]), "..", "conf.py")) as f:
        for line in f:
            if line.startswith("version = release = '"):
                ver = line.strip().split(" = ")[2].strip("'")
                git_tag = "v" + ver
                ver = ver.split(".")
                if len(ver) == 2:
                    ver.append("0")
                return git_tag, "<no hash>", ver
    return None


def make_version_header(filename):
    # Get version info using git, with fallback to docs/conf.py
    info = get_version_info_from_git()
    if info is None:
        info = get_version_info_from_docs_conf()

    git_tag, git_hash, ver = info
    if len(ver) < 3:
        ver = ("0", "0", "0")
        version_string = git_hash
    else:
        version_string = ".".join(ver)

    build_date = datetime.date.today()
    if "SOURCE_DATE_EPOCH" in os.environ:
        build_date = datetime.datetime.utcfromtimestamp(
            int(os.environ["SOURCE_DATE_EPOCH"])
        ).date()

    # Generate the file with the git and version info
    file_data = """\
// This file was generated by py/makeversionhdr.py
#define MICROPY_GIT_TAG "%s"
#define MICROPY_GIT_HASH "%s"
#define MICROPY_BUILD_DATE "%s"
#define MICROPY_VERSION_MAJOR (%s)
#define MICROPY_VERSION_MINOR (%s)
#define MICROPY_VERSION_MICRO (%s)
#define MICROPY_VERSION_STRING "%s"
#define MICROPY_FULL_VERSION_INFO ("Adafruit CircuitPython " MICROPY_GIT_TAG " on " MICROPY_BUILD_DATE "; " MICROPY_HW_BOARD_NAME " with " MICROPY_HW_MCU_NAME)
""" % (
        git_tag,
        git_hash,
        datetime.date.today().strftime("%Y-%m-%d"),
        ver[0].replace("v", ""),
        ver[1],
        ver[2],
        version_string,
    )

    # Check if the file contents changed from last time
    write_file = True
    if os.path.isfile(filename):
        with open(filename, "r") as f:
            existing_data = f.read()
        if existing_data == file_data:
            write_file = False

    # Only write the file if we need to
    if write_file:
        with open(filename, "w") as f:
            f.write(file_data)


if __name__ == "__main__":
    make_version_header(sys.argv[1])



# test errors from bad function calls

# function doesn't take keyword args
try:
    [].append(x=1)
except TypeError:
    print('TypeError')

# function with variable number of positional args given too few
try:
    round()
except TypeError:
    print('TypeError')

# function with variable number of positional args given too many
try:
    round(1, 2, 3)
except TypeError:
    print('TypeError')

# function with fixed number of positional args given wrong number
try:
    [].append(1, 2)
except TypeError:
    print('TypeError')

# function with keyword args given extra positional args
try:
    [].sort(1)
except TypeError:
    print('TypeError')

# function with keyword args given extra keyword args
try:
    [].sort(noexist=1)
except TypeError:
    print('TypeError')

# kw given for positional, but a different positional is missing
try:
    def f(x, y): pass
    f(x=1)
except TypeError:
    print('TypeError')



"""
categories: Modules,sys
description: Overriding sys.stdin, sys.stdout and sys.stderr not possible
cause: They are stored in read-only memory.
workaround: Unknown
"""
import sys

sys.stdin = None
print(sys.stdin)



"""
categories: Modules,deque
description: Deque not implemented
cause: Unknown
workaround: Use regular lists. micropython-lib has implementation of collections.deque.
"""
import collections

D = collections.deque()
print(D)



#!/usr/bin/python3


def defines(name, function):
    print(f"mcu_pin_function_t {name} [] = {{")
    for instance in (0, 1):
        for port in "ABCD":
            for idx in range(32):
                pin = f"P{port}{idx:02d}"
                pinmux = f"PINMUX_{pin}I_SDHC{instance}_{function}"
                print(
                    f"""\
#if defined({pinmux}) && ! defined(IGNORE_PIN_{pin})
    {{&pin_{pin}, {instance}, PIN_{pin}, {pinmux} & 0xffff}},
#endif"""
                )
    print(f"{{NULL, 0, 0}}")
    print(f"}};")
    print()


print(
    """\
#include <stdint.h>
#include "py/obj.h"
#include "sam.h"
#include "samd/pins.h"
#include "mpconfigport.h"
#include "atmel_start_pins.h"
#include "hal/include/hal_gpio.h"
#include "common-hal/microcontroller/Pin.h"

"""
)

defines("sdio_ck", "SDCK")
defines("sdio_cmd", "SDCMD")
defines("sdio_dat0", "SDDAT0")
defines("sdio_dat1", "SDDAT1")
defines("sdio_dat2", "SDDAT2")
defines("sdio_dat3", "SDDAT3")



# test syntax errors for uPy-specific decorators


def test_syntax(code):
    try:
        exec(code)
    except SyntaxError:
        print("SyntaxError")


# invalid micropython decorators
test_syntax("@micropython.a\ndef f(): pass")
test_syntax("@micropython.a.b\ndef f(): pass")



# Tests that the pending exception state is managed correctly
# (previously failed on native emitter).

def noop_task():
    print('noop task')
    yield 1

def raise_task():
    print('raise task')
    yield 2
    print('raising')
    raise Exception

def main():
    try:
        yield from raise_task()
    except:
        print('main exception')

    yield from noop_task()

for z in main():
    print('outer iter', z)



class CtxMgr:

    def __enter__(self):
        print("__enter__")
        return self

    def __exit__(self, a, b, c):
        print("__exit__", repr(a), repr(b))


with CtxMgr() as a:
    print(isinstance(a, CtxMgr))

try:
    with CtxMgr() as a:
        raise ValueError
except ValueError:
    print("ValueError")


class CtxMgr2:

    def __enter__(self):
        print("__enter__")
        return self

    def __exit__(self, a, b, c):
        print("__exit__", repr(a), repr(b))
        return True

try:
    with CtxMgr2() as a:
        raise ValueError
    print("No ValueError2")
except ValueError:
    print("ValueError2")


# These recursive try-finally tests are attempt to get some interpretation
# of last phrase in http://docs.python.org/3.4/library/dis.html#opcode-WITH_CLEANUP
# "If the stack represents an exception, and the function call returns a 'true'
# value, this information is "zapped" and replaced with a single WHY_SILENCED
# to prevent END_FINALLY from re-raising the exception. (But non-local gotos
# will still be resumed.)"
print("===")
with CtxMgr2() as a:
    try:
        try:
            raise ValueError
            print("No ValueError3")
        finally:
            print("finally1")
    finally:
        print("finally2")

print("===")
try:
    try:
        with CtxMgr2() as a:
            try:
                try:
                    raise ValueError
                    print("No ValueError3")
                finally:
                    print("finally1")
            finally:
                print("finally2")
    finally:
        print("finally3")
finally:
    print("finally4")



try:
    bytearray()[:] = bytearray()
except TypeError:
    print("SKIP")
    raise SystemExit

# test slices; only 2 argument version supported by MicroPython at the moment
x = bytearray(range(10))

# Assignment
l = bytearray(x)
l[1:3] = bytearray([10, 20])
print(l)
l = bytearray(x)
l[1:3] = bytearray([10])
print(l)
l = bytearray(x)
l[1:3] = bytearray()
print(l)
l = bytearray(x)
#del l[1:3]
print(l)

l = bytearray(x)
l[:3] = bytearray([10, 20])
print(l)
l = bytearray(x)
l[:3] = bytearray()
print(l)
l = bytearray(x)
#del l[:3]
print(l)

l = bytearray(x)
l[:-3] = bytearray([10, 20])
print(l)
l = bytearray(x)
l[:-3] = bytearray()
print(l)
l = bytearray(x)
#del l[:-3]
print(l)

# slice assignment that extends the array
b = bytearray(2)
b[2:] = bytearray(10)
print(b)

b = bytearray(10)
b[:-1] = bytearray(500)
print(len(b), b[0], b[-1])

# extension with self on RHS
b = bytearray(x)
b[4:] = b
print(b)

# Assignment of bytes to array slice
b = bytearray(2)
b[1:1] = b"12345"
print(b)

# Growth of bytearray via slice extension
b = bytearray(b'12345678')
b.append(57) # expand and add a bit of unused space at end of the bytearray
for i in range(400):
    b[-1:] = b'ab' # grow slowly into the unused space
print(len(b), b)



# test slice assignment to memoryview

try:
    memoryview(bytearray(1))[:] = memoryview(bytearray(1))
except (NameError, TypeError):
    print("SKIP")
    raise SystemExit

try:
    import uarray as array
except ImportError:
    try:
        import array
    except ImportError:
        print("SKIP")
        raise SystemExit

# test slice assignment between memoryviews
b1 = bytearray(b'1234')
b2 = bytearray(b'5678')
b3 = bytearray(b'5678')
m1 = memoryview(b1)
m2 = memoryview(b2)
m3 = memoryview(b3)
m2[1:3] = m1[0:2]
print(b2)
b3[1:3] = m1[0:2]
print(b3)
m1[2:4] = b3[1:3]
print(b1)

# invalid slice assignments
try:
    m2[1:3] = b1[0:4]
except ValueError:
    print("ValueError")
try:
    m2[1:3] = m1[0:4]
except ValueError:
    print("ValueError")
try:
    m2[0:4] = m1[1:3]
except ValueError:
    print("ValueError")

# test memoryview of arrays with items sized larger than 1
a1 = array.array('i', [0]*5)
m4 = memoryview(a1)
a2 = array.array('i', [3]*5)
m5 = memoryview(a2)
m4[1:3] = m5[1:3]
print(a1)

try:
    m4[1:3] = m2[1:3]
except ValueError:
    print("ValueError")

# invalid assignment on RHS
try:
    memoryview(array.array('i'))[0:2] = b'1234'
except ValueError:
    print('ValueError')

# test shift left of bytearray
b = bytearray(range(10))
mv = memoryview(b)
mv[1:] = mv[:-1]
print(b)

# test shift right of bytearray
b = bytearray(range(10))
mv = memoryview(b)
mv[:-1] = mv[1:]
print(b)

# test shift left of array
a = array.array('I', range(10))
mv = memoryview(a)
mv[1:] = mv[:-1]
print(a)

# test shift right of array
a = array.array('I', range(10))
mv = memoryview(a)
mv[:-1] = mv[1:]
print(a)



# list poppin'
a = [1, 2, 3]
print(a.pop())
print(a.pop())
print(a.pop())
try:
    print(a.pop())
except IndexError:
    print("IndexError raised")
else:
    raise AssertionError("No IndexError raised")

# popping such that list storage shrinks (tests implementation detail of uPy)
l = list(range(20))
for i in range(len(l)):
    l.pop()
print(l)



print(int(False))
print(int(True))

print(int(0))
print(int(1))
print(int(+1))
print(int(-1))

print(int('0'))
print(int('+0'))
print(int('-0'))
print(int('1'))
print(int('+1'))
print(int('-1'))
print(int('01'))
print(int('9'))
print(int('10'))
print(int('+10'))
print(int('-10'))
print(int('12'))
print(int('-12'))
print(int('99'))
print(int('100'))
print(int('314'))
print(int(' 314'))
print(int('314 '))
print(int('  \t\t  314  \t\t  '))
print(int('  1  '))
print(int(' -3 '))

print(int('0', 10))
print(int('1', 10))
print(int(' \t 1 \t ', 10))
print(int('11', 10))
print(int('11', 16))
print(int('11', 8))
print(int('11', 2))
print(int('11', 36))
print(int('xyz', 36))
print(int('0o123', 0))
print(int('8388607'))
print(int('0x123', 16))
print(int('0X123', 16))
print(int('0A', 16))
print(int('0o123', 8))
print(int('0O123', 8))
print(int('0123', 8))
print(int('0b100', 2))
print(int('0B100', 2))
print(int('0100', 2))
print(int(' \t 0o12', 8))
print(int('0o12  \t  ', 8))
print(int(b"12", 10))
print(int(b"12"))


def test(value, base):
    try:
        print(int(value, base))
    except ValueError:
        print('ValueError')


test('x', 0)
test('1x', 0)
test('  1x', 0)
test('  1' + chr(2) + '  ', 0)
test('', 0)
test(' ', 0)
test('  \t\t  ', 0)
test('0x', 16)
test('0x', 0)
test('0o', 8)
test('0o', 0)
test('0b', 2)
test('0b', 0)
test('0b2', 2)
test('0o8', 8)
test('0xg', 16)
test('1 1', 16)
test('123', 37)

# check that we don't parse this as a floating point number
print(0x1e+1)

# can't convert list to int
try:
    int([])
except TypeError:
    print("TypeError")



# MicroPython uasyncio module
# MIT license; Copyright (c) 2019-2020 Damien P. George

from . import core

# Event class for primitive events that can be waited on, set, and cleared
class Event:
    def __init__(self):
        self.state = False  # False=unset; True=set
        self.waiting = core.TaskQueue()  # Queue of Tasks waiting on completion of this event

    def is_set(self):
        return self.state

    def set(self):
        # Event becomes set, schedule any tasks waiting on it
        # Note: This must not be called from anything except the thread running
        # the asyncio loop (i.e. neither hard or soft IRQ, or a different thread).
        while self.waiting.peek():
            core._task_queue.push_head(self.waiting.pop_head())
        self.state = True

    def clear(self):
        self.state = False

    async def wait(self):
        if not self.state:
            # Event not set, put the calling task on the event's waiting queue
            self.waiting.push_head(core.cur_task)
            # Set calling task's data to the event's queue so it can be removed if needed
            core.cur_task.data = self.waiting
            yield
        return True



try:
    import ustruct as struct
except:
    try:
        import struct
    except ImportError:
        print("SKIP")
        raise SystemExit

# check maximum pack on 32-bit machine
print(struct.pack("<I", 2**32 - 1))
print(struct.pack("<I", 0xffffffff))

# long long ints
print(struct.pack("<Q", 1))
print(struct.pack(">Q", 1))
print(struct.pack("<Q", 2**64 - 1))
print(struct.pack(">Q", 2**64 - 1))
print(struct.pack("<Q", 0xffffffffffffffff))
print(struct.pack(">Q", 0xffffffffffffffff))
print(struct.pack("<q", -1))
print(struct.pack(">q", -1))
print(struct.pack("<Q", 1234567890123456789))
print(struct.pack("<q", -1234567890123456789))
print(struct.pack(">Q", 1234567890123456789))
print(struct.pack(">q", -1234567890123456789))
print(struct.unpack("<Q", b"\x12\x34\x56\x78\x90\x12\x34\x56"))
print(struct.unpack(">Q", b"\x12\x34\x56\x78\x90\x12\x34\x56"))
print(struct.unpack("<q", b"\x12\x34\x56\x78\x90\x12\x34\xf6"))
print(struct.unpack(">q", b"\xf2\x34\x56\x78\x90\x12\x34\x56"))

# check maximum unpack
print(struct.unpack("<I", b"\xff\xff\xff\xff"))
print(struct.unpack("<Q", b"\xff\xff\xff\xff\xff\xff\xff\xff"))

# check small int overflow
print(struct.unpack("<i", b'\xff\xff\xff\x7f'))
print(struct.unpack("<q", b'\xff\xff\xff\xff\xff\xff\xff\x7f'))

# test with negative big integers that are actually small in magnitude
bigzero = (1 << 70) - (1 << 70)
for endian in "<>":
    for type_ in "bhiq":
        fmt = endian + type_
        b = struct.pack(fmt, -2 + bigzero)
        print(fmt, b, struct.unpack(fmt, b))



print("Yep, I got imported.")

try:
    x = const(1)
except NameError:
    print("const not defined")

const = lambda x: x

_CNT01 = "CONST01"
_CNT02 = const(123)
A123 = const(123)
a123 = const(123)


def dummy():
    return False


def saysomething():
    print("There, I said it.")


def neverexecuted():
    print("Never got here!")


print("Yep, got here")



# test class with __getitem__, __setitem__, __delitem__ methods

class C:
    def __getitem__(self, item):
        print('get', item)
        return 'item'

    def __setitem__(self, item, value):
        print('set', item, value)

    def __delitem__(self, item):
        print('del', item)

c = C()
print(c[1])
c[1] = 2
del c[3]

# index not supported
class A:
    pass
a = A()
try:
    a[1]
except TypeError:
    print('TypeError')



# test builtin ord (whether or not we support unicode)

print(ord('a'))

try:
    ord('')
except TypeError:
    print("TypeError")

# bytes also work in ord

print(ord(b'a'))
print(ord(b'\x00'))
print(ord(b'\x01'))
print(ord(b'\x7f'))
print(ord(b'\x80'))
print(ord(b'\xff'))

try:
    ord(b'')
except TypeError:
    print("TypeError")

# argument must be a string
try:
    ord(1)
except TypeError:
    print('TypeError')



try:
    str.partition
except AttributeError:
    print("SKIP")
    raise SystemExit

print("asdf".rpartition('g'))
print("asdf".rpartition('a'))
print("asdf".rpartition('s'))
print("asdf".rpartition('f'))
print("asdf".rpartition('d'))
print("asdf".rpartition('asd'))
print("asdf".rpartition('sdf'))
print("asdf".rpartition('as'))
print("asdf".rpartition('df'))
print("asdf".rpartition('asdf'))
print("asdf".rpartition('asdfa'))
print("asdf".rpartition('fasdf'))
print("asdf".rpartition('fasdfa'))
print("abba".rpartition('a'))
print("abba".rpartition('b'))

try:
    print("asdf".rpartition(1))
except TypeError:
    print("Raised TypeError")
else:
    print("Did not raise TypeError")

try:
    print("asdf".rpartition(''))
except ValueError:
    print("Raised ValueError")
else:
    print("Did not raise ValueError")



# This file is part of the MicroPython project, http://micropython.org/
#
# The MIT License (MIT)
#
# SPDX-FileCopyrightText: Copyright (c) 2020 Lucian Copeland for Adafruit Industries
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import csv
import sys

# Use: parse_pins_csv.py Filename.csv
# Designed for use with .csv files from Micropython, or in identical format
# created via Datasheet peripheral tables with a Sheets program.
#
# See examples/nucleo_h743.csv for example formatting.

# Open target file
with open(sys.argv[1]) as csv_file:
    csv_reader = csv.reader(csv_file, delimiter=",")
    line_count = 0

    print("STATIC const mp_rom_map_elem_t board_module_globals_table[] = {")

    for row in csv_reader:
        label = row[0]
        pin = row[1]
        if len(pin) < 4:
            pin = pin[:2] + "0" + pin[2:]
        print("{ MP_ROM_QSTR(MP_QSTR_" + label + "), MP_ROM_PTR(&pin_" + pin + ") },")
        line_count += 1

    print(f"Processed {line_count} lines.")



# SPDX-FileCopyrightText: 2014 MicroPython & CircuitPython contributors (https://github.com/adafruit/circuitpython/graphs/contributors)
#
# SPDX-License-Identifier: MIT

import sys
import mistune
import mistune.renderers

print(sys.argv[1])

with open(sys.argv[1], "r") as source_file:
    source = source_file.read()

html = mistune.create_markdown()

print()
print("HTML")
print("=====================================")
print('From the <a href="">GitHub release page</a>:\n<blockquote>')
print(html(source))
print("</blockquote>")


class AdafruitBBCodeRenderer(mistune.renderers.BaseRenderer):
    def placeholder(self):
        return ""

    def paragraph(self, text):
        return text + "\n\n"

    def block_text(self, text):
        return text

    def text(self, text):
        return text

    def link(self, link, title, text):
        return "[url={}]{}[/url]".format(link, title)

    def autolink(self, link, is_email):
        if not is_email:
            return "[url={}]{}[/url]".format(link, link)
        return link

    def heading(self, text, level):
        return "[b][size=150]{}[/size][/b]\n".format(text)

    def codespan(self, text):
        return "[color=#E74C3C][size=95]{}[/size][/color]".format(text)

    def list_item(self, text, level):
        return "[*]{}[/*]\n".format(text.strip())

    def list(self, text, ordered, level, start=None):
        ordered_indicator = "=" if ordered else ""
        return "[list{}]\n{}[/list]".format(ordered_indicator, text)

    def double_emphasis(self, text):
        return "[b]{}[/b]".format(text)

    def emphasis(self, text):
        return "[b]{}[/b]".format(text)

    def strong(self, text):
        return "[i]{}[/i]".format(text)


bbcode = mistune.create_markdown(renderer=AdafruitBBCodeRenderer())

print()
print("BBCode")
print("=====================================")
print("From the [url=]GitHub release page[/url]:\n[quote]")
print(bbcode(source))
print("[/quote]")



# check modulo matches python definition

# This tests compiler version
print(123 // 7)
print(-123 // 7)
print(123 // -7)
print(-123 // -7)

a = 10000001
b = 10000000
print(a // b)
print(a // -b)
print(-a // b)
print(-a // -b)



# Test for VfsPosix

try:
    import uos

    uos.VfsPosix
except (ImportError, AttributeError):
    print("SKIP")
    raise SystemExit

# We need a directory for testing that doesn't already exist.
# Skip the test if it does exist.
temp_dir = "micropy_test_dir"
try:
    uos.stat(temp_dir)
    print("SKIP")
    raise SystemExit
except OSError:
    pass

# getcwd and chdir
curdir = uos.getcwd()
uos.chdir("/")
print(uos.getcwd())
uos.chdir(curdir)
print(uos.getcwd() == curdir)

# stat
print(type(uos.stat("/")))

# listdir and ilistdir
print(type(uos.listdir("/")))

# mkdir
uos.mkdir(temp_dir)

# file create
f = open(temp_dir + "/test", "w")
f.write("hello")
f.close()

# close on a closed file should succeed
f.close()

# construct a file object using the type constructor, with a raw fileno
f = type(f)(2)
print(f)

# file read
f = open(temp_dir + "/test", "r")
print(f.read())
f.close()

# rename
uos.rename(temp_dir + "/test", temp_dir + "/test2")
print(uos.listdir(temp_dir))

# construct new VfsPosix with path argument
vfs = uos.VfsPosix(temp_dir)
print(list(i[0] for i in vfs.ilistdir(".")))

# stat, statvfs
print(type(vfs.stat(".")))
print(type(vfs.statvfs(".")))

# check types of ilistdir with str/bytes arguments
print(type(list(vfs.ilistdir("."))[0][0]))
print(type(list(vfs.ilistdir(b"."))[0][0]))

# remove
uos.remove(temp_dir + "/test2")
print(uos.listdir(temp_dir))

# remove with error
try:
    uos.remove(temp_dir + "/test2")
except OSError:
    print("remove OSError")

# rmdir
uos.rmdir(temp_dir)
print(temp_dir in uos.listdir())

# rmdir with error
try:
    uos.rmdir(temp_dir)
except OSError:
    print("rmdir OSError")



# Tests that the correct nested exception handler is used when
# throwing into a generator (previously failed on native emitter).

def gen():
    try:
        yield 1
        try:
            yield 2
            try:
                yield 3
            except Exception:
                yield 4
                print(0)
            yield 5
        except Exception:
            yield 6
            print(1)
        yield 7
    except Exception:
        yield 8
        print(2)
    yield 9

for i in range(1, 10):
    g = gen()
    try:
        for _ in range(i):
            print(next(g))
        print(g.throw(ValueError))
    except ValueError:
        print('ValueError')
    except StopIteration:
        print('StopIteration')



# test attributes of builtin range type

try:
    range(0).start
except AttributeError:
    print("SKIP")
    raise SystemExit

# attrs
print(range(1, 2, 3).start)
print(range(1, 2, 3).stop)
print(range(1, 2, 3).step)

# bad attr (can't store)
try:
    range(4).start = 0
except AttributeError:
    print('AttributeError')



# Test various loop types, some may be implemented/optimized differently
while True:
    try:
        break
    finally:
        print('finally 1')


for i in [1, 5, 10]:
    try:
        continue
    finally:
        print('finally 2')

for i in range(3):
    try:
        continue
    finally:
        print('finally 3')

# Multi-level
for i in range(4):
    print(i)
    try:
        while True:
            try:
                try:
                    break
                finally:
                    print('finally 1')
            finally:
                print('finally 2')
        print('here')
    finally:
        print('finnaly 3')

# break from within try-finally, within for-loop
for i in [1]:
    try:
        print(i)
        break
    finally:
        print('finally 4')

# Test unwind-jump where there is nothing in the body of the try or finally.
# This checks that the bytecode emitter allocates enough stack for the unwind.
for i in [1]:
    try:
        break
    finally:
        pass

# The following test checks that the globals dict is valid after a call to a
# function that has an unwind jump.
# There was a bug where an unwind jump would trash the globals dict upon return
# from a function, because it used the Python-stack incorrectly.
def f():
    for i in [1]:
        try:
            break
        finally:
            pass
def g():
    global global_var
    f()
    print(global_var)
global_var = 'global'
g()



# nested exceptions

def f():
    try:
        foo()
    except:
        print("except 1")
        try:
            baz()
        except:
            print("except 2")
        bar()

try:
    f()
except:
    print("f except")



# test super() operations which don't require allocation
import micropython

# Check for stackless build, which can't call functions without
# allocating a frame on heap.
try:

    def stackless():
        pass

    micropython.heap_lock()
    stackless()
    micropython.heap_unlock()
except RuntimeError:
    print("SKIP")
    raise SystemExit


class A:
    def foo(self):
        print("A foo")
        return 42


class B(A):
    def foo(self):
        print("B foo")
        print(super().foo())


b = B()

micropython.heap_lock()
b.foo()
micropython.heap_unlock()



class Base1:
    def __init__(self, *args):
        print("Base1.__init__", args)

class Ctuple1(Base1, tuple):
    pass

a = Ctuple1()
print(len(a))
a = Ctuple1([1, 2, 3])
print(len(a))

print("---")

class Ctuple2(tuple, Base1):
    pass

a = Ctuple2()
print(len(a))
a = Ctuple2([1, 2, 3])
print(len(a))

a = tuple([1,2,3])
b = Ctuple1([1,2,3])
c = Ctuple2([1,2,3])

print(a == b)
print(b == c)
print(c == a)



"""
categories: Types,bytes
description: bytes() with keywords not implemented
cause: Unknown
workaround: Pass the encoding as a positional parameter, e.g. ``print(bytes('abc', 'utf-8'))``
"""
print(bytes("abc", encoding="utf8"))



# test threads sleeping
#
# SPDX-FileCopyrightText: Copyright (c) 2016 Damien P. George on behalf of Pycom Ltd
#
# SPDX-License-Identifier: MIT

try:
    import utime

    sleep_ms = utime.sleep_ms
except ImportError:
    import time

    sleep_ms = lambda t: time.sleep(t / 1000)

import _thread

lock = _thread.allocate_lock()
n_thread = 4
n_finished = 0


def thread_entry(t):
    global n_finished
    sleep_ms(t)
    sleep_ms(2 * t)
    with lock:
        n_finished += 1


for i in range(n_thread):
    _thread.start_new_thread(thread_entry, (10 * i,))

# wait for threads to finish
while n_finished < n_thread:
    sleep_ms(100)
print("done", n_thread)



try:
    import uhashlib as hashlib
except ImportError:
    try:
        import hashlib
    except ImportError:
        # This is neither uPy, nor cPy, so must be uPy with
        # uhashlib module disabled.
        print("SKIP")
        raise SystemExit

try:
    hashlib.md5
except AttributeError:
    # MD5 is only available on some ports
    print("SKIP")
    raise SystemExit

md5 = hashlib.md5(b"hello")
md5.update(b"world")
print(md5.digest())



# test builtin print function

print("break the test")
print()
print(None)
print('')
print(1)
print(1, 2)

print(sep='')
print(sep='x')
print(end='')
print(end='x\n')
print(1, sep='')
print(1, end='')
print(1, sep='', end='')
print(1, 2, sep='')
print(1, 2, end='')
print(1, 2, sep='', end='')

print([{1:2}])



# Test that tasks return their value correctly to the caller

try:
    import uasyncio as asyncio
except ImportError:
    try:
        import asyncio
    except ImportError:
        print("SKIP")
        raise SystemExit


async def foo():
    return 42


async def main():
    # Call function directly via an await
    print(await foo())

    # Create a task and await on it
    task = asyncio.create_task(foo())
    print(await task)


asyncio.run(main())



#!/usr/bin/env python3
import socket

TIMEOUT = 10
HOST = "192.168.10.179"
PORT = 5000

print("Create Socket")
with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
    s.settimeout(TIMEOUT)
    s.bind((HOST, PORT))
    s.listen()
    print("Accepting connections")
    while True:
        try:
            conn, addr = s.accept()
            break
        except BlockingIOError:
            pass

    with conn:
        s.settimeout(TIMEOUT)
        print("Connected by", addr)
        data = conn.recv(128)
        print("got: " + str(data))
        conn.sendall(data)
        print("sent: " + str(data))



# test loading from ptr32 type


@micropython.viper
def get(src: ptr32) -> int:
    return src[0]


@micropython.viper
def get1(src: ptr32) -> int:
    return src[1]


@micropython.viper
def memadd(src: ptr32, n: int) -> int:
    sum = 0
    for i in range(n):
        sum += src[i]
    return sum


@micropython.viper
def memadd2(src_in) -> int:
    src = ptr32(src_in)
    n = int(len(src_in)) >> 2
    sum = 0
    for i in range(n):
        sum += src[i]
    return sum


b = bytearray(b"\x12\x12\x12\x12\x34\x34\x34\x34")
print(b)
print(hex(get(b)), hex(get1(b)))
print(hex(memadd(b, 2)))
print(hex(memadd2(b)))



from ... import mod1
from ...mod2 import bar

print(mod1.foo)
print(bar)

# attempted relative import beyond top-level package
try:
    from .... import mod1
except ImportError:
    print("ImportError")



(x := 4)
print(x)

if x := 2:
    print(True)
print(x)

print(4, x := 5)
print(x)

x = 1
print(x, x := 5, x)
print(x)


def foo():
    print("any", any((hit := i) % 5 == 3 and (hit % 2) == 0 for i in range(10)))
    return hit


hit = 123
print(foo())
print(hit)  # shouldn't be changed by foo

print("any", any((hit := i) % 5 == 3 and (hit % 2) == 0 for i in range(10)))
print(hit)  # should be changed by above

print([((m := k + 1), k * m) for k in range(4)])
print(m)



# test for type.__bases__ implementation

if not hasattr(object, '__bases__'):
    print("SKIP")
    raise SystemExit

class A:
    pass

class B(object):
    pass

class C(B):
    pass

class D(C, A):
    pass

# Check the attribute exists
print(hasattr(A, '__bases__'))
print(hasattr(B, '__bases__'))
print(hasattr(C, '__bases__'))
print(hasattr(D, '__bases__'))

# Check it is always a tuple
print(type(A.__bases__) == tuple)
print(type(B.__bases__) == tuple)
print(type(C.__bases__) == tuple)
print(type(D.__bases__) == tuple)

# Check size
print(len(A.__bases__) == 1)
print(len(B.__bases__) == 1)
print(len(C.__bases__) == 1)
print(len(D.__bases__) == 2)

# Check values
print(A.__bases__[0] == object)
print(B.__bases__[0] == object)
print(C.__bases__[0] == B)
print(D.__bases__[0] == C)
print(D.__bases__[1] == A)

# Object has an empty tuple
print(object.__bases__ == tuple())



try:
    from utime import ticks_diff, ticks_add
except ImportError:
    print("SKIP")
    raise SystemExit

MAX = ticks_add(0, -1)
# Should be done like this to avoid small int overflow
MODULO_HALF = MAX // 2 + 1

# Invariants:
# if ticks_diff(a, b) = c,
# then ticks_diff(b, a) = -c

assert ticks_diff(1, 0) == 1, ticks_diff(1, 0)
assert ticks_diff(0, 1) == -1

assert ticks_diff(0, MAX) == 1
assert ticks_diff(MAX, 0) == -1

assert ticks_diff(0, MAX - 1) == 2

# Maximum "positive" distance
assert ticks_diff(MODULO_HALF, 1) == MODULO_HALF - 1, ticks_diff(MODULO_HALF, 1)
# Step further, and it becomes a negative distance
assert ticks_diff(MODULO_HALF, 0) == -MODULO_HALF

# Offsetting that in either direction doesn't affect the result
off = 100
# Cheating and skipping to use ticks_add() when we know there's no wraparound
# Real apps should use always it.
assert ticks_diff(MODULO_HALF + off, 1 + off) == MODULO_HALF - 1
assert ticks_diff(MODULO_HALF + off, 0 + off) == -MODULO_HALF
assert ticks_diff(MODULO_HALF - off, ticks_add(1, -off)) == MODULO_HALF - 1
assert ticks_diff(MODULO_HALF - off, ticks_add(0, -off)) == -MODULO_HALF

print("OK")



# This file is part of the MicroPython project, http://micropython.org/
#
# The MIT License (MIT)
#
# SPDX-FileCopyrightText: Copyright (c) 2020 Lucian Copeland for Adafruit Industries
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import csv
import sys

# Use: parse_af_csf.py Filename.csv -pins-only
# Designed for use with .csv files from Micropython, or in identical format
# created via Datasheet peripheral tables with a Sheets program.
#
# See examples/stm32f405.csv for example formatting.

# Most peripherals (SPI, I2C) output 3 values:
# peripheral index, alt function, pin string
def evaluate_periph(inper, inlist, periph, subtype, altfn, pin):
    # ex) SPI1_SCK,SPI3_SCK/I2S3_CK
    # Clean anything after a '\' due to SPI/I2S mixing
    if not inper.find("/") == -1:
        inper = inper[: inper.find("/")]

    if inper[: len(periph)] == periph and inper[-len(subtype) :] == subtype:
        inlist.append([inper[len(periph) : len(periph) + 1], altfn, pin])


# Timers (TIM) are a special case with 4 values
# timer index, alt function, channel, pin string
def evaluate_tim(inper, inlist, altfn, pin):
    # ex) TIM2_CH1/TIM2_ETR,  TIM5_CH1
    # Clean anything after a '\' to filter ETR
    if not inper.find("/") == -1:
        inper = inper[: inper.find("/")]

    if inper[:3] == "TIM" and inper[5:7] == "CH" and inper[-1:] != "N":
        inlist.append([inper[3:4], altfn, inper[-1:], pin])
    elif inper[:3] == "TIM" and inper[6:8] == "CH" and inper[-1:] != "N":
        inlist.append([inper[3:5], altfn, inper[-1:], pin])


# Open target file
with open(sys.argv[1]) as csv_file:
    csv_reader = csv.reader(csv_file, delimiter=",")
    line_count = 0

    if sys.argv[2] != "-pins-only":
        # List of peripheral pin types to read
        todo = [
            ["I2C", "SDA"],
            ["I2C", "SCL"],
            ["SPI", "SCK"],
            ["SPI", "MOSI"],
            ["SPI", "MISO"],
            ["UART", "TX"],
            ["UART", "RX"],
        ]

        # Make a list of empty lists to populate
        outlist = []
        for items in todo:
            empty = []
            outlist.append(empty)
        # TIM
        empty = []
        outlist.append(empty)

        # Each line is a list of strings
        for row in csv_reader:
            altfn = 0
            pin = row[1]
            if len(pin) < 4:
                pin = pin[:2] + "0" + pin[2:]
            for col in row:
                array_index = 0
                # Evaluate the string for every possible todo entry
                for item in todo:
                    evaluate_periph(col, outlist[array_index], item[0], item[1], altfn - 2, pin)
                    # UART special case, run again for USART variant
                    if item[0] == "UART":
                        evaluate_periph(
                            col, outlist[array_index], "USART", item[1], altfn - 2, pin
                        )
                    array_index += 1
                # TIM special case
                evaluate_tim(col, outlist[-1], altfn - 2, pin)
                altfn += 1
            line_count += 1

        # Print formatted output
        for i in range(len(todo)):
            ins = (todo[i][0]).lower() + "_" + (todo[i][1]).lower() + "_"
            #      const mcu_i2c_sda_obj_t mcu_i2c_sda_list[4] = {
            print("const mcu_periph_obj_t mcu_" + ins + "list[" + str(len(outlist[i])) + "] = {")
            for row in outlist[i]:
                print("    PERIPH(" + row[0] + ", " + str(row[1]) + ", &pin_" + row[2] + "),")
            print("};")

        # Timer special case:
        print("const mcu_tim_pin_obj_t mcu_tim_pin_list[" + str(len(outlist[-1])) + "] = {")
        for row in outlist[-1]:
            print(
                "    TIM("
                + row[0]
                + ", "
                + str(row[1])
                + ", "
                + str(row[2])
                + ", &pin_"
                + row[3]
                + "),"
            )
        print("};")

    else:
        # Format all viable pins listed
        # ex) const mcu_pin_obj_t pin_PE02 = PIN(4, 2, NO_ADC);
        # ex 2) extern const mcu_pin_obj_t pin_PE02;
        # TODO: add ADC detection
        outlist = []

        for row in csv_reader:
            altfn = 0
            pin = row[1]
            if len(pin) < 4:
                pin = pin[:2] + "0" + pin[2:]
            outlist.append([pin, str(ord(row[1][1:2]) - 65), row[1][2:4]])
            line_count += 1

        for line in outlist:
            print(
                "const mcu_pin_obj_t pin_"
                + line[0]
                + " = PIN("
                + line[1]
                + ", "
                + line[2]
                + ", NO_ADC);"
            )

        for line in outlist:
            print("extern const mcu_pin_obj_t pin_" + line[0] + ";")

    print(f"Processed {line_count} lines.")



try:
    memoryview(b'a').itemsize
except:
    print("SKIP")
    raise SystemExit
try:
    from uarray import array
except ImportError:
    try:
        from array import array
    except ImportError:
        print("SKIP")
        raise SystemExit

for code in ['b', 'h', 'i', 'q', 'f', 'd']:
    print(memoryview(array(code)).itemsize)

# 'l' varies depending on word size of the machine
print(memoryview(array('l')).itemsize in (4, 8))

# shouldn't be able to store to the itemsize attribute
try:
    memoryview(b'a').itemsize = 1
except AttributeError:
    print('AttributeError')



# test ustruct and endian specific things

try:
    import ustruct as struct
except:
    try:
        import struct
    except ImportError:
        print("SKIP")
        raise SystemExit

# unpack/unpack_from with unaligned native type
buf = b'0123456789'
print(struct.unpack('h', memoryview(buf)[1:3]))
print(struct.unpack_from('i', buf, 1))
print(struct.unpack_from('@i', buf, 1))
print(struct.unpack_from('@ii', buf, 1))

# pack_into with unaligned native type
buf = bytearray(b'>----<<<<<<<')
struct.pack_into('i', buf, 1, 0x30313233)
print(buf)
struct.pack_into('@ii', buf, 3, 0x34353637, 0x41424344)
print(buf)



# test importing of .mpy files with native code (x64 only)

try:
    import sys, uio, uos

    uio.IOBase
    uos.mount
except (ImportError, AttributeError):
    print("SKIP")
    raise SystemExit

if not (sys.platform == "linux" and sys.maxsize > 2 ** 32):
    print("SKIP")
    raise SystemExit


class UserFile(uio.IOBase):
    def __init__(self, data):
        self.data = memoryview(data)
        self.pos = 0

    def readinto(self, buf):
        n = min(len(buf), len(self.data) - self.pos)
        buf[:n] = self.data[self.pos : self.pos + n]
        self.pos += n
        return n

    def ioctl(self, req, arg):
        return 0


class UserFS:
    def __init__(self, files):
        self.files = files

    def mount(self, readonly, mksfs):
        pass

    def umount(self):
        pass

    def stat(self, path):
        if path in self.files:
            return (32768, 0, 0, 0, 0, 0, 0, 0, 0, 0)
        raise OSError

    def open(self, path, mode):
        return UserFile(self.files[path])


# these are the test .mpy files
# fmt: off
user_files = {
    # bad architecture
    '/mod0.mpy': b'C\x05\xff\x00\x10',

    # test loading of viper and asm
    '/mod1.mpy': (
        b'C\x05\x0b\x1f\x20' # header

        b'\x20' # n bytes, bytecode
            b'\x00\x08\x02m\x02m' # prelude
            b'\x51' # LOAD_CONST_NONE
            b'\x63' # RETURN_VALUE

            b'\x00\x02' # n_obj, n_raw_code

        b'\x22' # n bytes, viper code
            b'\x00\x00\x00\x00\x00\x00' # dummy machine code
            b'\x00\x00' # qstr0
            b'\x01\x0c\x0aprint' # n_qstr, qstr0
            b'\x00\x00\x00' # scope_flags, n_obj, n_raw_code

        b'\x23' # n bytes, asm code
            b'\x00\x00\x00\x00\x00\x00\x00\x00' # dummy machine code
            b'\x00\x00\x00' # scope_flags, n_pos_args, type_sig
    ),

    # test loading viper with additional scope flags and relocation
    '/mod2.mpy': (
        b'C\x05\x0b\x1f\x20' # header

        b'\x20' # n bytes, bytecode
            b'\x00\x08\x02m\x02m' # prelude
            b'\x51' # LOAD_CONST_NONE
            b'\x63' # RETURN_VALUE

            b'\x00\x01' # n_obj, n_raw_code

        b'\x12' # n bytes(=4), viper code
            b'\x00\x00\x00\x00' # dummy machine code
            b'\x00' # n_qstr
            b'\x81\x60' # scope_flags: VIPERBSS | VIPERRODATA | VIPERRELOC
            b'\x00\x00' # n_obj, n_raw_code
            b'\x06rodata' # rodata, 6 bytes
            b'\x04' # bss, 4 bytes
            b'\x03\x01\x00' # dummy relocation of rodata
    ),
}
# fmt: on

# create and mount a user filesystem
uos.mount(UserFS(user_files), "/userfs")
sys.path.append("/userfs")

# import .mpy files from the user filesystem
for i in range(len(user_files)):
    mod = "mod%u" % i
    try:
        __import__(mod)
        print(mod, "OK")
    except ValueError as er:
        print(mod, "ValueError", er)

# unmount and undo path addition
uos.umount("/userfs")
sys.path.pop()



# test construction of bytes from different objects

# tuple, list
print(bytes((1, 2)))
print(bytes([1, 2]))

# constructor value out of range
try:
    bytes([-1])
except ValueError:
    print('ValueError')

# constructor value out of range
try:
    bytes([256])
except ValueError:
    print('ValueError')

# error in construction
try:
    a = bytes([1, 2, 3], 1)
except TypeError:
    print('TypeError')



# locals referenced before assignment

def f1():
    print(x)
    x = 1

def f2():
    for i in range(0):
        print(i)
    print(i)

def check(f):
    try:
        f()
    except NameError:
        print("NameError")

check(f1)
check(f2)



# test float formatting

# general rounding
for val in (116, 1111, 1234, 5010, 11111):
    print("%.0f" % val)
    print("%.1f" % val)
    print("%.3f" % val)

# make sure rounding is done at the correct precision
for prec in range(8):
    print(("%%.%df" % prec) % 6e-5)

# check certain cases that had a digit value of 10 render as a ":" character
print("%.2e" % float("9" * 51 + "e-39"))
print("%.2e" % float("9" * 40 + "e-21"))

# check a case that would render negative digit values, eg ")" characters
# the string is converted back to a float to check for no illegal characters
float("%.23e" % 1e-80)



try:
    import ujson as json
    from ucollections import OrderedDict
except ImportError:
    try:
        import json
        from collections import OrderedDict
    except ImportError:
        print("SKIP")
        raise SystemExit

print(json.dumps(OrderedDict(((1, 2), (3, 4)))))



# test loading from ptr16 type
# only works on little endian machines


@micropython.viper
def get(src: ptr16) -> int:
    return src[0]


@micropython.viper
def get1(src: ptr16) -> int:
    return src[1]


@micropython.viper
def memadd(src: ptr16, n: int) -> int:
    sum = 0
    for i in range(n):
        sum += src[i]
    return sum


@micropython.viper
def memadd2(src_in) -> int:
    src = ptr16(src_in)
    n = int(len(src_in)) >> 1
    sum = 0
    for i in range(n):
        sum += src[i]
    return sum


b = bytearray(b"1234")
print(b)
print(get(b), get1(b))
print(memadd(b, 2))
print(memadd2(b))



# unpack sequences

a, = 1,     ; print(a)
a, b = 2, 3 ; print(a, b)
a, b, c = 1, 2, 3; print(a, b, c)

a, = range(1); print(a)
a, b = range(2); print(a, b)
a, b, c = range(3); print(a, b, c)

(a) = range(1); print(a)
(a,) = range(1); print(a)
(a, b) = range(2); print(a, b)
(a, b, c) = range(3); print(a, b, c)
(a, (b, c)) = [-1, range(2)]; print(a, b, c)

# lists

[] = []
[a] = range(1); print(a)
[a, b] = range(2); print(a, b)
[a, b, c] = range(3); print(a, b, c)

# with star

*a, = ()    ; print(a)
*a, = 4,    ; print(a)
*a, = 5, 6  ; print(a)

*a, b = 7,          ; print(a, b)
*a, b = 8, 9        ; print(a, b)
*a, b = 10, 11, 12  ; print(a, b)

a, *b = 13,         ; print(a, b)
a, *b = 14, 15      ; print(a, b)
a, *b = 16, 17, 18  ; print(a, b)

a, *b, c = 19, 20           ; print(a, b)
a, *b, c = 21, 22, 23       ; print(a, b)
a, *b, c = 24, 25, 26, 27   ; print(a, b)

a = [28, 29]
*b, = a
print(a, b, a == b, a is b)

[*a] = [1, 2, 3]
print(a)

try:
    a, *b, c = (30,)
except ValueError:
    print("ValueError")

# with star and generic iterator

*a, = range(5)          ; print(a)
*a, b = range(5)        ; print(a, b)
*a, b, c = range(5)     ; print(a, b, c)
a, *b = range(5)        ; print(a, b)
a, *b, c = range(5)     ; print(a, b, c)
a, *b, c, d = range(5)  ; print(a, b, c, d)
a, b, *c = range(5)     ; print(a, b, c)
a, b, *c, d = range(5)  ; print(a, b, c, d)
a, b, *c, d, e = range(5)   ; print(a, b, c, d, e)

*a, = [x * 2 for x in [1, 2, 3, 4]]     ; print(a)
*a, b = [x * 2 for x in [1, 2, 3, 4]]   ; print(a, b)
a, *b = [x * 2 for x in [1, 2, 3, 4]]   ; print(a, b)
a, *b, c = [x * 2 for x in [1, 2, 3, 4]]; print(a, b, c)

try:
    a, *b, c = range(0)
except ValueError:
    print("ValueError")

try:
    a, *b, c = range(1)
except ValueError:
    print("ValueError")



# Pure Python AES encryption routines.
#
# AES is integer based and inplace so doesn't use the heap.  It is therefore
# a good test of raw performance and correctness of the VM/runtime.
#
# The AES code comes first (code originates from a C version authored by D.P.George)
# and then the test harness at the bottom.
#
# MIT license; Copyright (c) 2016 Damien P. George on behalf of Pycom Ltd

##################################################################
# discrete arithmetic routines, mostly from a precomputed table

# non-linear, invertible, substitution box
# fmt: off
aes_s_box_table = bytes((
    0x63,0x7c,0x77,0x7b,0xf2,0x6b,0x6f,0xc5,0x30,0x01,0x67,0x2b,0xfe,0xd7,0xab,0x76,
    0xca,0x82,0xc9,0x7d,0xfa,0x59,0x47,0xf0,0xad,0xd4,0xa2,0xaf,0x9c,0xa4,0x72,0xc0,
    0xb7,0xfd,0x93,0x26,0x36,0x3f,0xf7,0xcc,0x34,0xa5,0xe5,0xf1,0x71,0xd8,0x31,0x15,
    0x04,0xc7,0x23,0xc3,0x18,0x96,0x05,0x9a,0x07,0x12,0x80,0xe2,0xeb,0x27,0xb2,0x75,
    0x09,0x83,0x2c,0x1a,0x1b,0x6e,0x5a,0xa0,0x52,0x3b,0xd6,0xb3,0x29,0xe3,0x2f,0x84,
    0x53,0xd1,0x00,0xed,0x20,0xfc,0xb1,0x5b,0x6a,0xcb,0xbe,0x39,0x4a,0x4c,0x58,0xcf,
    0xd0,0xef,0xaa,0xfb,0x43,0x4d,0x33,0x85,0x45,0xf9,0x02,0x7f,0x50,0x3c,0x9f,0xa8,
    0x51,0xa3,0x40,0x8f,0x92,0x9d,0x38,0xf5,0xbc,0xb6,0xda,0x21,0x10,0xff,0xf3,0xd2,
    0xcd,0x0c,0x13,0xec,0x5f,0x97,0x44,0x17,0xc4,0xa7,0x7e,0x3d,0x64,0x5d,0x19,0x73,
    0x60,0x81,0x4f,0xdc,0x22,0x2a,0x90,0x88,0x46,0xee,0xb8,0x14,0xde,0x5e,0x0b,0xdb,
    0xe0,0x32,0x3a,0x0a,0x49,0x06,0x24,0x5c,0xc2,0xd3,0xac,0x62,0x91,0x95,0xe4,0x79,
    0xe7,0xc8,0x37,0x6d,0x8d,0xd5,0x4e,0xa9,0x6c,0x56,0xf4,0xea,0x65,0x7a,0xae,0x08,
    0xba,0x78,0x25,0x2e,0x1c,0xa6,0xb4,0xc6,0xe8,0xdd,0x74,0x1f,0x4b,0xbd,0x8b,0x8a,
    0x70,0x3e,0xb5,0x66,0x48,0x03,0xf6,0x0e,0x61,0x35,0x57,0xb9,0x86,0xc1,0x1d,0x9e,
    0xe1,0xf8,0x98,0x11,0x69,0xd9,0x8e,0x94,0x9b,0x1e,0x87,0xe9,0xce,0x55,0x28,0xdf,
    0x8c,0xa1,0x89,0x0d,0xbf,0xe6,0x42,0x68,0x41,0x99,0x2d,0x0f,0xb0,0x54,0xbb,0x16,
))
# fmt: on

# multiplication of polynomials modulo x^8 + x^4 + x^3 + x + 1 = 0x11b
def aes_gf8_mul_2(x):
    if x & 0x80:
        return (x << 1) ^ 0x11B
    else:
        return x << 1


def aes_gf8_mul_3(x):
    return x ^ aes_gf8_mul_2(x)


# non-linear, invertible, substitution box
def aes_s_box(a):
    return aes_s_box_table[a & 0xFF]


# return 0x02^(a-1) in GF(2^8)
def aes_r_con(a):
    ans = 1
    while a > 1:
        ans <<= 1
        if ans & 0x100:
            ans ^= 0x11B
        a -= 1
    return ans


##################################################################
# basic AES algorithm; see FIPS-197

# all inputs must be size 16
def aes_add_round_key(state, w):
    for i in range(16):
        state[i] ^= w[i]


# combined sub_bytes, shift_rows, mix_columns, add_round_key
# all inputs must be size 16
def aes_sb_sr_mc_ark(state, w, w_idx, temp):
    temp_idx = 0
    for i in range(4):
        x0 = aes_s_box_table[state[i * 4]]
        x1 = aes_s_box_table[state[1 + ((i + 1) & 3) * 4]]
        x2 = aes_s_box_table[state[2 + ((i + 2) & 3) * 4]]
        x3 = aes_s_box_table[state[3 + ((i + 3) & 3) * 4]]
        temp[temp_idx] = aes_gf8_mul_2(x0) ^ aes_gf8_mul_3(x1) ^ x2 ^ x3 ^ w[w_idx]
        temp[temp_idx + 1] = x0 ^ aes_gf8_mul_2(x1) ^ aes_gf8_mul_3(x2) ^ x3 ^ w[w_idx + 1]
        temp[temp_idx + 2] = x0 ^ x1 ^ aes_gf8_mul_2(x2) ^ aes_gf8_mul_3(x3) ^ w[w_idx + 2]
        temp[temp_idx + 3] = aes_gf8_mul_3(x0) ^ x1 ^ x2 ^ aes_gf8_mul_2(x3) ^ w[w_idx + 3]
        w_idx += 4
        temp_idx += 4
    for i in range(16):
        state[i] = temp[i]


# combined sub_bytes, shift_rows, add_round_key
# all inputs must be size 16
def aes_sb_sr_ark(state, w, w_idx, temp):
    temp_idx = 0
    for i in range(4):
        x0 = aes_s_box_table[state[i * 4]]
        x1 = aes_s_box_table[state[1 + ((i + 1) & 3) * 4]]
        x2 = aes_s_box_table[state[2 + ((i + 2) & 3) * 4]]
        x3 = aes_s_box_table[state[3 + ((i + 3) & 3) * 4]]
        temp[temp_idx] = x0 ^ w[w_idx]
        temp[temp_idx + 1] = x1 ^ w[w_idx + 1]
        temp[temp_idx + 2] = x2 ^ w[w_idx + 2]
        temp[temp_idx + 3] = x3 ^ w[w_idx + 3]
        w_idx += 4
        temp_idx += 4
    for i in range(16):
        state[i] = temp[i]


# take state as input and change it to the next state in the sequence
# state and temp have size 16, w has size 16 * (Nr + 1), Nr >= 1
def aes_state(state, w, temp, nr):
    aes_add_round_key(state, w)
    w_idx = 16
    for i in range(nr - 1):
        aes_sb_sr_mc_ark(state, w, w_idx, temp)
        w_idx += 16
    aes_sb_sr_ark(state, w, w_idx, temp)


# expand 'key' to 'w' for use with aes_state
# key has size 4 * Nk, w has size 16 * (Nr + 1), temp has size 16
def aes_key_expansion(key, w, temp, nk, nr):
    for i in range(4 * nk):
        w[i] = key[i]
    w_idx = 4 * nk - 4
    for i in range(nk, 4 * (nr + 1)):
        t = temp
        t_idx = 0
        if i % nk == 0:
            t[0] = aes_s_box(w[w_idx + 1]) ^ aes_r_con(i // nk)
            for j in range(1, 4):
                t[j] = aes_s_box(w[w_idx + (j + 1) % 4])
        elif nk > 6 and i % nk == 4:
            for j in range(0, 4):
                t[j] = aes_s_box(w[w_idx + j])
        else:
            t = w
            t_idx = w_idx
        w_idx += 4
        for j in range(4):
            w[w_idx + j] = w[w_idx + j - 4 * nk] ^ t[t_idx + j]


##################################################################
# simple use of AES algorithm, using output feedback (OFB) mode


class AES:
    def __init__(self, keysize):
        if keysize == 128:
            self.nk = 4
            self.nr = 10
        elif keysize == 192:
            self.nk = 6
            self.nr = 12
        else:
            assert keysize == 256
            self.nk = 8
            self.nr = 14

        self.state = bytearray(16)
        self.w = bytearray(16 * (self.nr + 1))
        self.temp = bytearray(16)
        self.state_pos = 16

    def set_key(self, key):
        aes_key_expansion(key, self.w, self.temp, self.nk, self.nr)
        self.state_pos = 16

    def set_iv(self, iv):
        for i in range(16):
            self.state[i] = iv[i]
        self.state_pos = 16

    def get_some_state(self, n_needed):
        if self.state_pos >= 16:
            aes_state(self.state, self.w, self.temp, self.nr)
            self.state_pos = 0
        n = 16 - self.state_pos
        if n > n_needed:
            n = n_needed
        return n

    def apply_to(self, data):
        idx = 0
        n = len(data)
        while n > 0:
            ln = self.get_some_state(n)
            n -= ln
            for i in range(ln):
                data[idx + i] ^= self.state[self.state_pos + i]
            idx += ln
            self.state_pos += n


###########################################################################
# Benchmark interface

bm_params = {
    (50, 25): (1, 16),
    (100, 100): (1, 32),
    (1000, 1000): (4, 256),
    (5000, 1000): (20, 256),
}


def bm_setup(params):
    nloop, datalen = params

    aes = AES(256)
    key = bytearray(256 // 8)
    iv = bytearray(16)
    data = bytearray(datalen)
    # from now on we don't use the heap

    def run():
        for loop in range(nloop):
            # encrypt
            aes.set_key(key)
            aes.set_iv(iv)
            for i in range(2):
                aes.apply_to(data)

            # decrypt
            aes.set_key(key)
            aes.set_iv(iv)
            for i in range(2):
                aes.apply_to(data)

            # verify
            for i in range(len(data)):
                assert data[i] == 0

    def result():
        return params[0] * params[1], True

    return run, result



# test store to ptr32 type


@micropython.viper
def set(dest: ptr32, val: int):
    dest[0] = val


@micropython.viper
def set1(dest: ptr32, val: int):
    dest[1] = val


@micropython.viper
def memset(dest: ptr32, val: int, n: int):
    for i in range(n):
        dest[i] = val


@micropython.viper
def memset2(dest_in, val: int):
    dest = ptr32(dest_in)
    n = int(len(dest_in)) >> 2
    for i in range(n):
        dest[i] = val


b = bytearray(8)
print(b)

set(b, 0x42424242)
print(b)

set1(b, 0x43434343)
print(b)

memset(b, 0x44444444, len(b) // 4)
print(b)

memset2(b, 0x45454545)
print(b)



# test basic capability to start a new thread
#
# SPDX-FileCopyrightText: Copyright (c) 2016 Damien P. George on behalf of Pycom Ltd
#
# SPDX-License-Identifier: MIT

try:
    import utime as time
except ImportError:
    import time
import _thread


def foo():
    pass


def thread_entry(n):
    for i in range(n):
        foo()


for i in range(2):
    while True:
        try:
            _thread.start_new_thread(thread_entry, ((i + 1) * 10,))
            break
        except OSError:
            pass

# wait for threads to finish
time.sleep(1)
print("done")



"""Non-volatile memory

The `nvm` module allows you to store whatever raw bytes you wish in a
reserved section non-volatile memory.

Note that this module can't be imported and used directly. The sole
instance of :class:`ByteArray` is available at
:attr:`microcontroller.nvm`."""

from __future__ import annotations

from typing import overload

from _typing import ReadableBuffer

class ByteArray:
    r"""Presents a stretch of non-volatile memory as a bytearray.

    Non-volatile memory is available as a byte array that persists over reloads
    and power cycles. Each assignment causes an erase and write cycle so its recommended to assign
    all values to change at once.

    Usage::

       import microcontroller
       microcontroller.nvm[0:3] = b"\xcc\x10\x00"
    """
    def __init__(self) -> None:
        """Not currently dynamically supported. Access the sole instance through `microcontroller.nvm`."""
        ...
    def __bool__(self) -> bool: ...
    def __len__(self) -> int:
        """Return the length. This is used by (`len`)"""
        ...
    @overload
    def __getitem__(self, index: slice) -> bytearray: ...
    @overload
    def __getitem__(self, index: int) -> int:
        """Returns the value at the given index."""
        ...
    @overload
    def __setitem__(self, index: slice, value: ReadableBuffer) -> None: ...
    @overload
    def __setitem__(self, index: int, value: int) -> None:
        """Set the value at the given index."""
        ...



# Inspired by urlparse.py from CPython 3.3 stdlib
# There was a bug in MicroPython that under some conditions class stored
# in instance attribute later was returned "bound" as if it was a method,
# which caused class constructor to receive extra argument.
try:
    from collections import namedtuple
except ImportError:
    print("SKIP")
    raise SystemExit

import skip_if
skip_if.no_cpython_compat()

_DefragResultBase = namedtuple('DefragResult', [ 'foo', 'bar' ])

class _ResultMixinStr(object):
    def encode(self):
        return self._encoded_counterpart(*(x.encode() for x in self))

class _ResultMixinBytes(object):
    def decode(self):
        return self._decoded_counterpart(*(x.decode() for x in self))

class DefragResult(_DefragResultBase, _ResultMixinStr):
    pass

class DefragResultBytes(_DefragResultBase, _ResultMixinBytes):
    pass


DefragResult._encoded_counterpart = DefragResultBytes
DefragResultBytes._decoded_counterpart = DefragResult

# Due to differences in type and native subclass printing,
# the best thing we can do here is to just test that no exceptions
# happen

#print(DefragResult, DefragResult._encoded_counterpart)
#print(DefragResultBytes, DefragResultBytes._decoded_counterpart)

o1 = DefragResult("a", "b")
#print(o1, type(o1))
o2 = DefragResultBytes("a", "b")
#print(o2, type(o2))

#print(o1._encoded_counterpart)
_o1 = o1.encode()
print(_o1[0], _o1[1])
#print(_o1, type(_o1))

print("All's ok")



# test builtin hash function with float args

# these should hash to an integer with a specific value
for val in (
    "0.0",
    "-0.0",
    "1.0",
    "2.0",
    "-12.0",
    "12345.0",
):
    print(val, hash(float(val)))

# just check that these values are hashable
for val in (
    "0.1",
    "-0.1",
    "10.3",
    "0.4e3",
    "1e16",
    "inf",
    "-inf",
    "nan",
):
    print(val, type(hash(float(val))))



# Test that int.from_bytes() for small number of bytes generates
# small int.
import micropython

micropython.heap_lock()
print(int.from_bytes(b"1", "little"))
print(int.from_bytes(b"12", "little"))
print(int.from_bytes(b"2\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", "little"))
micropython.heap_unlock()



# Test for VfsLittle using a RAM device, testing error handling from corrupt block device

try:
    import uos

    uos.VfsLfs1
    uos.VfsLfs2
except (ImportError, AttributeError):
    print("SKIP")
    raise SystemExit


class RAMBlockDevice:
    ERASE_BLOCK_SIZE = 1024

    def __init__(self, blocks):
        self.data = bytearray(blocks * self.ERASE_BLOCK_SIZE)
        self.ret = 0

    def readblocks(self, block, buf, off):
        addr = block * self.ERASE_BLOCK_SIZE + off
        for i in range(len(buf)):
            buf[i] = self.data[addr + i]
        return self.ret

    def writeblocks(self, block, buf, off):
        addr = block * self.ERASE_BLOCK_SIZE + off
        for i in range(len(buf)):
            self.data[addr + i] = buf[i]
        return self.ret

    def ioctl(self, op, arg):
        if op == 4:  # block count
            return len(self.data) // self.ERASE_BLOCK_SIZE
        if op == 5:  # block size
            return self.ERASE_BLOCK_SIZE
        if op == 6:  # erase block
            return 0


def corrupt(bdev, block):
    addr = block * bdev.ERASE_BLOCK_SIZE
    for i in range(bdev.ERASE_BLOCK_SIZE):
        bdev.data[addr + i] = i & 0xFF


def create_vfs(bdev, vfs_class):
    bdev.ret = 0
    vfs_class.mkfs(bdev)
    vfs = vfs_class(bdev)
    with vfs.open("f", "w") as f:
        for i in range(100):
            f.write("test")
    return vfs


def test(bdev, vfs_class):
    print("test", vfs_class)

    # statvfs
    vfs = create_vfs(bdev, vfs_class)
    corrupt(bdev, 0)
    corrupt(bdev, 1)
    try:
        print(vfs.statvfs(""))
    except OSError:
        print("statvfs OSError")

    # error during read
    vfs = create_vfs(bdev, vfs_class)
    f = vfs.open("f", "r")
    bdev.ret = -5  # EIO
    try:
        f.read(10)
    except OSError:
        print("read OSError")

    # error during write
    vfs = create_vfs(bdev, vfs_class)
    f = vfs.open("f", "a")
    bdev.ret = -5  # EIO
    try:
        f.write("test")
    except OSError:
        print("write OSError")

    # error during close
    vfs = create_vfs(bdev, vfs_class)
    f = vfs.open("f", "w")
    f.write("test")
    bdev.ret = -5  # EIO
    try:
        f.close()
    except OSError:
        print("close OSError")

    # error during flush
    vfs = create_vfs(bdev, vfs_class)
    f = vfs.open("f", "w")
    f.write("test")
    bdev.ret = -5  # EIO
    try:
        f.flush()
    except OSError:
        print("flush OSError")
    bdev.ret = 0
    f.close()


bdev = RAMBlockDevice(30)
test(bdev, uos.VfsLfs1)
test(bdev, uos.VfsLfs2)



# parser tests

try:
    compile
except NameError:
    print("SKIP")
    raise SystemExit

# completely empty string
# uPy and CPy differ for this case
#try:
#    compile("", "stdin", "single")
#except SyntaxError:
#    print("SyntaxError")
try:
    compile("", "stdin", "eval")
except SyntaxError:
    print("SyntaxError")
compile("", "stdin", "exec")

# empty continued line
try:
    compile("\\\n", "stdin", "single")
except SyntaxError:
    print("SyntaxError")
try:
    compile("\\\n", "stdin", "eval")
except SyntaxError:
    print("SyntaxError")
compile("\\\n", "stdin", "exec")



# case where generator doesn't intercept the thrown/injected exception
def gen():
    yield 123
    yield 456

g = gen()
print(next(g))
try:
    g.throw(KeyError)
except KeyError:
    print('got KeyError from downstream!')

# case where a thrown exception is caught and stops the generator
def gen():
    try:
        yield 1
        yield 2
    except:
        pass
g = gen()
print(next(g))
try:
    g.throw(ValueError)
except StopIteration:
    print('got StopIteration')

# generator ignores a thrown GeneratorExit (this is allowed)
def gen():
    try:
        yield 123
    except GeneratorExit as e:
        print('GeneratorExit', repr(e.args))
    yield 456

# thrown a class
g = gen()
print(next(g))
print(g.throw(GeneratorExit))

# thrown an instance
g = gen()
print(next(g))
print(g.throw(GeneratorExit()))

# thrown an instance with None as second arg
g = gen()
print(next(g))
print(g.throw(GeneratorExit(), None))

# thrown a class and instance
g = gen()
print(next(g))
print(g.throw(GeneratorExit, GeneratorExit(123)))



# test parsing of floats, requiring double-precision

# very large integer part with a very negative exponent should cancel out
print(float("9" * 400 + "e-100"))
print(float("9" * 400 + "e-200"))
print(float("9" * 400 + "e-400"))

# many fractional digits
print(float("." + "9" * 400))
print(float("." + "9" * 400 + "e100"))
print(float("." + "9" * 400 + "e-100"))

# tiny fraction with large exponent
print("%.14e" % float("." + "0" * 400 + "9e100"))
print("%.14e" % float("." + "0" * 400 + "9e200"))
print("%.14e" % float("." + "0" * 400 + "9e400"))

# ensure that accuracy is retained when value is close to a subnormal
print(float("1.00000000000000000000e-307"))
print(float("10.0000000000000000000e-308"))
print(float("100.000000000000000000e-309"))



"""Support for audio output

The `audioio` module contains classes to provide access to audio IO.

All classes change hardware state and should be deinitialized when they
are no longer needed if the program continues after use. To do so, either
call :py:meth:`!deinit` or use a context manager. See
:ref:`lifetime-and-contextmanagers` for more info.

Since CircuitPython 5, `RawSample` and `WaveFile` are moved
to :mod:`audiocore`, and `Mixer` is moved to :mod:`audiomixer`.

For compatibility with CircuitPython 4.x, some builds allow the items in
`audiocore` to be imported from `audioio`.  This will be removed for all
boards in a future build of CircuitPython."""

from __future__ import annotations

from typing import Optional

import _typing
import microcontroller

class AudioOut:
    """Output an analog audio signal"""

    def __init__(
        self,
        left_channel: microcontroller.Pin,
        *,
        right_channel: Optional[microcontroller.Pin] = None,
        quiescent_value: int = 0x8000
    ) -> None:
        """Create a AudioOut object associated with the given pin(s). This allows you to
        play audio signals out on the given pin(s).

        :param ~microcontroller.Pin left_channel: The pin to output the left channel to
        :param ~microcontroller.Pin right_channel: The pin to output the right channel to
        :param int quiescent_value: The output value when no signal is present. Samples should start
            and end with this value to prevent audible popping.

        Simple 8ksps 440 Hz sin wave::

          import audiocore
          import audioio
          import board
          import array
          import time
          import math

          # Generate one period of sine wav.
          length = 8000 // 440
          sine_wave = array.array("H", [0] * length)
          for i in range(length):
              sine_wave[i] = int(math.sin(math.pi * 2 * i / length) * (2 ** 15) + 2 ** 15)

          dac = audioio.AudioOut(board.SPEAKER)
          sine_wave = audiocore.RawSample(sine_wave, sample_rate=8000)
          dac.play(sine_wave, loop=True)
          time.sleep(1)
          dac.stop()

        Playing a wave file from flash::

          import board
          import audioio
          import digitalio

          # Required for CircuitPlayground Express
          speaker_enable = digitalio.DigitalInOut(board.SPEAKER_ENABLE)
          speaker_enable.switch_to_output(value=True)

          data = open("cplay-5.1-16bit-16khz.wav", "rb")
          wav = audiocore.WaveFile(data)
          a = audioio.AudioOut(board.A0)

          print("playing")
          a.play(wav)
          while a.playing:
            pass
          print("stopped")"""
        ...
    def deinit(self) -> None:
        """Deinitialises the AudioOut and releases any hardware resources for reuse."""
        ...
    def __enter__(self) -> AudioOut:
        """No-op used by Context Managers."""
        ...
    def __exit__(self) -> None:
        """Automatically deinitializes the hardware when exiting a context. See
        :ref:`lifetime-and-contextmanagers` for more info."""
        ...
    def play(self, sample: _typing.AudioSample, *, loop: bool = False) -> None:
        """Plays the sample once when loop=False and continuously when loop=True.
        Does not block. Use `playing` to block.

        Sample must be an `audiocore.WaveFile`, `audiocore.RawSample`, `audiomixer.Mixer` or `audiomp3.MP3Decoder`.

        The sample itself should consist of 16 bit samples. Microcontrollers with a lower output
        resolution will use the highest order bits to output. For example, the SAMD21 has a 10 bit
        DAC that ignores the lowest 6 bits when playing 16 bit samples."""
        ...
    def stop(self) -> None:
        """Stops playback and resets to the start of the sample."""
        ...
    playing: bool
    """True when an audio sample is being output even if `paused`. (read-only)"""
    def pause(self) -> None:
        """Stops playback temporarily while remembering the position. Use `resume` to resume playback."""
        ...
    def resume(self) -> None:
        """Resumes sample playback after :py:func:`pause`."""
        ...
    paused: bool
    """True when playback is paused. (read-only)"""



# very basic test of ssl module, just to test the methods exist

try:
    import uio as io
    import ussl as ssl
except ImportError:
    print("SKIP")
    raise SystemExit

# create in client mode
try:
    ss = ssl.wrap_socket(io.BytesIO(), server_hostname="test.example.com")
except OSError as er:
    print("wrap_socket:", repr(er))

# create in server mode (can use this object for further tests)
socket = io.BytesIO()
ss = ssl.wrap_socket(socket, server_side=1)

# print
print(repr(ss)[:12])

# setblocking() propagates call to the underlying stream object, and
# io.BytesIO doesn't have setblocking() (in CPython too).
# try:
#    ss.setblocking(False)
# except NotImplementedError:
#    print('setblocking: NotImplementedError')
# ss.setblocking(True)

# write
print(ss.write(b"aaaa"))

# read (underlying socket has no data)
print(ss.read(8))

# read (underlying socket has data, but it's bad data)
socket.write(b"aaaaaaaaaaaaaaaa")
socket.seek(0)
try:
    ss.read(8)
except OSError as er:
    print("read:", repr(er))

# close
ss.close()
# close 2nd time
ss.close()

# read on closed socket
try:
    ss.read(10)
except OSError as er:
    print("read:", repr(er))

# write on closed socket
try:
    ss.write(b"aaaa")
except OSError as er:
    print("write:", repr(er))



"""Displays text in a TileGrid

The `terminalio` module contains classes to display a character stream on a display. The built
in font is available as ``terminalio.FONT``."""

from __future__ import annotations

from typing import Optional

import displayio
import fontio
from _typing import ReadableBuffer

FONT: fontio.BuiltinFont
"""The built in font"""

class Terminal:
    """Display a character stream with a TileGrid"""

    def __init__(self, tilegrid: displayio.TileGrid, font: fontio.BuiltinFont) -> None:
        """Terminal manages tile indices and cursor position based on VT100 commands. The font should be
        a `fontio.BuiltinFont` and the TileGrid's bitmap should match the font's bitmap."""
        ...
    def write(self, buf: ReadableBuffer) -> Optional[int]:
        """Write the buffer of bytes to the bus.

        :return: the number of bytes written
        :rtype: int or None"""
        ...



try:
    frozenset
except NameError:
    print("SKIP")
    raise SystemExit

# Examples from https://docs.python.org/3/library/stdtypes.html#set
# "Instances of set are compared to instances of frozenset based on their
# members. For example:"
print(set('abc') == frozenset('abc'))
# This doesn't work in uPy
#print(set('abc') in set([frozenset('abc')]))



import audiocore
import audiopwmio
import board
import digitalio
import array
import time
import math

trigger = digitalio.DigitalInOut(board.D4)
trigger.switch_to_output(True)

# Generate one period of sine wav.
length = 8000 // 440

samples = []
sample_names = ["unsigned 8 bit", "signed 8 bit", "unsigned 16 bit", "signed 16 bit"]


# unsigned 8 bit
u8 = array.array("B", [0] * length)
for i in range(length):
    u8[i] = int(math.sin(math.pi * 2 * i / length) * (2 ** 7) + 2 ** 7)

samples.append(audiocore.RawSample(u8, sample_rate=4000))

# signed 8 bit
s8 = array.array("b", [0] * length)
for i in range(length):
    s8[i] = int(math.sin(math.pi * 2 * i / length) * (2 ** 7))

samples.append(audiocore.RawSample(s8, sample_rate=16000))

# unsigned 16 bit
u16 = array.array("H", [0] * length)
for i in range(length):
    u16[i] = int(math.sin(math.pi * 2 * i / length) * (2 ** 15) + 2 ** 15)

samples.append(audiocore.RawSample(u16, sample_rate=8000))


# signed 16 bit
s16 = array.array("h", [0] * length)
for i in range(length):
    s16[i] = int(math.sin(math.pi * 2 * i / length) * (2 ** 15))

samples.append(audiocore.RawSample(s16, sample_rate=8000))


dac = audiopwmio.PWMAudioOut(board.D13)
for sample, name in zip(samples, sample_names):
    print(name)
    trigger.value = False
    dac.play(sample, loop=True)
    time.sleep(1)
    dac.stop()
    time.sleep(0.1)
    trigger.value = True
    print()

print("done")



# default separator (whitespace)
print("a b".rsplit())
#print("   a   b    ".rsplit(None))
#print("   a   b    ".rsplit(None, 1))
#print("   a   b    ".rsplit(None, 2))
#print("   a   b  c  ".rsplit(None, 1))
#print("   a   b  c  ".rsplit(None, 0))
#print("   a   b  c  ".rsplit(None, -1))

# empty separator should fail (this actually delegates to .split())
try:
    "abc".rsplit('')
except ValueError:
    print("ValueError")

# empty separator should fail (error handled in .rsplit())
try:
    'a a a a'.rsplit('', 5)
except ValueError:
    print('ValueError')

# bad separator type
try:
    'a a a a'.rsplit(1)
except TypeError:
    print('TypeError')

# non-empty separator
print("abc".rsplit("a"))
print("abc".rsplit("b"))
print("abc".rsplit("c"))
print("abc".rsplit("z"))
print("abc".rsplit("ab"))
print("abc".rsplit("bc"))
print("abc".rsplit("abc"))
print("abc".rsplit("abcd"))
print("abcabc".rsplit("bc"))
print("abcabc".rsplit("bc", 0))
print("abcabc".rsplit("bc", 1))
print("abcabc".rsplit("bc", 2))

print("10/11/12".rsplit("/", 1))
print("10/11/12".rsplit("/", 2))
print("10/11/12".rsplit("/", 3))
print("10/11/12".rsplit("/", 4))
print("10/11/12".rsplit("/", 5))

print("/*10/*11/*12/*".rsplit("/*", 1))
print("/*10/*11/*12/*".rsplit("/*", 2))
print("/*10/*11/*12/*".rsplit("/*", 3))
print("/*10/*11/*12/*".rsplit("/*", 4))
print("/*10/*11/*12/*".rsplit("/*", 5))

print(b"abcabc".rsplit(b"bc", 2))

# negative "maxsplit" should delegate to .split()
print('abaca'.rsplit('a', -1))
print('abaca'.rsplit('a', -2))



import os

# Directory that the project lives in, aka ../..
SITE_ROOT = '/'.join(os.path.dirname(__file__).split('/')[0:-2])

TEMPLATE_DIRS = (
    "%s/templates/" % SITE_ROOT, # Your custom template directory, before the RTD one to override it.
    "%s/readthedocs/templates/" % SITE_ROOT, # Default RTD template dir
)



# test while conditions which are optimised by the compiler

while 0:
    print(0)
else:
    print(1)

while 1:
    print(2)
    break

while 2:
    print(3)
    break

while -1:
    print(4)
    break

while False:
    print('a')
else:
    print('b')

while True:
    print('a')
    break

while not False:
    print('a')
    break

while not True:
    print('a')
else:
    print('b')



# test builtin slice attributes access

# print slice attributes
class A:
    def __getitem__(self, idx):
        print(idx.start, idx.stop, idx.step)

try:
    t = A()[1:2]
except:
    print("SKIP")
    raise SystemExit


A()[1:2:3]

# test storing to attr (shouldn't be allowed)
class B:
    def __getitem__(self, idx):
        try:
            idx.start = 0
        except AttributeError:
            print('AttributeError')
B()[:]



print(dict([(1, "foo")]))
d = dict([("foo", "foo2"), ("bar", "baz")])
print(sorted(d.keys()))
print(sorted(d.values()))

try:
    dict(((1,),))
except ValueError:
    print("ValueError")

try:
    dict(((1, 2, 3),))
except ValueError:
    print("ValueError")



# cmdline: -v -v -v
# test printing of the parse-tree

for i in ():
    pass
a = None
b = "str"
c = "a very long str that will not be interned"
d = b"bytes"
e = b"a very long bytes that will not be interned"
f = 123456789012345678901234567890
g = 123



# test non-blocking UDP sockets

try:
    import usocket as socket, uerrno as errno
except ImportError:
    try:
        import socket, errno
    except ImportError:
        print("SKIP")
        raise SystemExit


s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
s.bind(socket.getaddrinfo("127.0.0.1", 8000)[0][-1])
s.settimeout(0)

try:
    s.recv(1)
except OSError as er:
    print("EAGAIN:", er.errno == errno.EAGAIN)
